Package: annotatr
Title: Annotation of Genomic Regions to Genomic Annotations
Version: 1.37.0
Date: 2025-06-21
Authors@R: c(
    person("Raymond G.", "Cavalcante", email = "rcavalca@umich.edu", role = c("aut", "cre")),
    person(c("Maureen A."), "Sartor", email = "sartorma@med.umich.edu", role = c("ths")))
Description: Given a set of genomic sites/regions (e.g. ChIP-seq peaks, CpGs, differentially methylated CpGs or regions, SNPs, etc.) it is often of interest to investigate the intersecting genomic annotations. Such annotations include those relating to gene models (promoters, 5'UTRs, exons, introns, and 3'UTRs), CpGs (CpG islands, CpG shores, CpG shelves), or regulatory sequences such as enhancers. The annotatr package provides an easy way to summarize and visualize the intersection of genomic sites/regions with genomic annotations.
Depends: R (>= 3.5.0)
Imports: AnnotationDbi, AnnotationHub, dplyr, GenomicFeatures (>=
        1.61.4), GenomicRanges (>= 1.61.1), Seqinfo, ggplot2 (>=
        3.5.0), IRanges, methods, readr, regioneR, reshape2, rlang,
        rtracklayer (>= 1.69.1), S4Vectors (>= 0.23.10), stats, utils
Suggests: GenomeInfoDb, BiocStyle, devtools, knitr, org.Dm.eg.db,
        org.Gg.eg.db, org.Hs.eg.db, org.Mm.eg.db, org.Rn.eg.db,
        rmarkdown, roxygen2, testthat,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        TxDb.Dmelanogaster.UCSC.dm6.ensGene,
        TxDb.Drerio.UCSC.danRer10.refGene,
        TxDb.Drerio.UCSC.danRer11.refGene,
        TxDb.Ggallus.UCSC.galGal5.refGene,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Mmusculus.UCSC.mm9.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene,
        TxDb.Mmusculus.UCSC.mm39.knownGene,
        TxDb.Rnorvegicus.UCSC.rn4.ensGene,
        TxDb.Rnorvegicus.UCSC.rn5.refGene,
        TxDb.Rnorvegicus.UCSC.rn6.refGene,
        TxDb.Rnorvegicus.UCSC.rn7.refGene
VignetteBuilder: knitr
BugReports: https://www.github.com/rcavalcante/annotatr/issues
License: GPL-3
LazyData: true
RoxygenNote: 7.3.3
Encoding: UTF-8
biocViews: Software, Annotation, GenomeAnnotation, FunctionalGenomics,
        Visualization
git_url: https://git.bioconductor.org/packages/annotatr
git_branch: devel
git_last_commit: 3e6ad83
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-07
NeedsCompilation: no
Packaged: 2025-11-07 21:14:08 UTC; biocbuild
Author: Raymond G. Cavalcante [aut, cre],
  Maureen A. Sartor [ths]
Maintainer: Raymond G. Cavalcante <rcavalca@umich.edu>

# Generated by roxygen2: do not edit by hand

export(annotate_regions)
export(annotatr_cache)
export(build_ah_annots)
export(build_annotations)
export(builtin_annotations)
export(builtin_genomes)
export(expand_annotations)
export(plot_annotation)
export(plot_categorical)
export(plot_coannotations)
export(plot_numerical)
export(plot_numerical_coannotations)
export(randomize_regions)
export(read_annotations)
export(read_regions)
export(subset_order_tbl)
export(summarize_annotations)
export(summarize_categorical)
export(summarize_numerical)
export(tidy_annotations)
import(AnnotationDbi, except='select')
import(AnnotationHub, except='query')
import(GenomicFeatures)
import(GenomicRanges, except=c('union','setdiff','intersect','union'))
import(dplyr)
import(ggplot2)
import(methods)
importClassesFrom(S4Vectors,Hits)
importClassesFrom(S4Vectors,Rle)
importClassesFrom(Seqinfo,Seqinfo)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,splitAsList)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqnames)
importFrom(readr,read_tsv)
importFrom(regioneR,randomizeRegions)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(rtracklayer,import)
importFrom(rtracklayer,import.bed)
importFrom(stats,as.formula)
importFrom(utils,combn)
importFrom(utils,data)

#' Read genomic regions in BEDX+Y format
#'
#' \code{read_regions()} reads genomic regions by calling the \code{rtracklayer::import()} function. This function can automatically deal with BEDX files from BED3 to BED6. For BED6+Y, the \code{extraCols} argument should be used to correctly interpret the extra columns.
#'
#' NOTE: The \code{name} (4th) and \code{score} (5th) columns are so named. If these columns have a particular meaning for your data, they should be renamed with the \code{rename_name} and/or \code{rename_score} parameters.
#'
#' @param con A path, URL, connection or BEDFile object. See \code{rtracklayer::import()} documentation.
#' @param genome From \code{rtracklayer::import()}: The identifier of a genome, or NA if unknown. Typically, this is a UCSC identifier like 'hg19'. An attempt will be made to derive the \code{seqinfo} on the return value using either an installed BSgenome package or UCSC, if network access is available.
#' @param format From \code{rtracklayer::import()}: The format of the output. If not missing, should be one of 'bed', 'bed15', 'bedGraph' or 'bedpe'. If missing and 'con' is a filename, the format is derived from the file extension. This argument is unnecessary when 'con' is a derivative of 'RTLFile'.
#' @param extraCols From \code{rtracklayer::import()}: A character vector in the same form as 'colClasses' from 'read.table'.  It should indicate the name and class of each extra/special column to read from the BED file. As BED does not encode column names, these are assumed to be the last columns in the file. This enables parsing of the various BEDX+Y formats.
#' @param rename_name A string to rename the name column of the BED file. For example, if the name column actually contains a categorical variable.
#' @param rename_score A string to rename the score column of the BED file. For example, if the score column represents a quantity about the data besides the score in the BED specification.
#' @param ... Parameters to pass onto the format-specific method of \code{rtracklayer::import()}.
#'
#' @return A \code{GRanges} object.
#'
#' @examples
#'
#'    # Example of reading a BED6+3 file where the last 3 columns are non-standard
#'    file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
#'    extraCols = c(diff_meth = 'numeric', mu0 = 'numeric', mu1 = 'numeric')
#'    gr = read_regions(con = file, genome = 'hg19', extraCols = extraCols, format = 'bed',
#'        rename_name = 'DM_status', rename_score = 'pval')
#'
#' @export
read_regions = function(con, genome = NA, format, extraCols = character(), rename_name, rename_score, ...) {

    if(!missing(format)) {
        gr = rtracklayer::import(con = con, genome = genome, format = format, extraCols = extraCols, ...)
    } else {
        gr = rtracklayer::import(con = con, genome = genome, extraCols = extraCols, ...)
    }

    # Rename name and score columns if the user desires
    if(!missing(rename_name)) {
        if(any(colnames(GenomicRanges::mcols(gr)) == 'name')) {
            colnames(GenomicRanges::mcols(gr))[which(colnames(GenomicRanges::mcols(gr)) == 'name')] = rename_name
        } else {
            warning('Ignoring rename_name parameter because con has no name column.')
        }
    }
    if(!missing(rename_score)) {
        if(any(colnames(GenomicRanges::mcols(gr)) == 'score')) {
            colnames(GenomicRanges::mcols(gr))[which(colnames(GenomicRanges::mcols(gr)) == 'score')] = rename_score
        } else {
            warning('Ignoring rename_score parameter because con has no score column.')
        }
    }

    return(gr)
}

#' Read custom annotations
#'
#' \code{read_annotations()} is a wrapper for the \code{rtracklayer::import()} function that creates a \code{GRanges} object matching the structure of annotations built with \code{build_annotations()}. The structure is defined by \code{GRanges}, with the \code{mcols()} with names \code{c('id','gene_id','symbol','type')}.
#'
#' @param con A path, URL, connection or BEDFile object. See \code{rtracklayer::import.bed()} documentation.
#' @param name A string for the name of the annotations to be used in the name of the object, [genome]_custom_[name]
#' @param genome From \code{rtracklayer::import()}: The identifier of a genome, or NA if unknown. Typically, this is a UCSC identifier like 'hg19'. An attempt will be made to derive the \code{seqinfo} on the return value using either an installed BSgenome package or UCSC, if network access is available.
#' @param format From \code{rtracklayer::import()}: The format of the output. If not missing, should be one of 'bed', 'bed15', 'bedGraph' or 'bedpe'. If missing and 'con' is a filename, the format is derived from the file extension. This argument is unnecessary when 'con' is a derivative of 'RTLFile'.
#' @param extraCols From \code{rtracklayer::import.bed()}: A character vector in the same form as 'colClasses' from 'read.table'.  It should indicate the name and class of each extra/special column to read from the BED file. As BED does not encode column names, these are assumed to be the last columns in the file. This enables parsing of the various BEDX+Y formats.
#' @param ... Parameters to pass onto the format-specific method of \code{rtracklayer::import()}.
#'
#' @return A \code{GRanges} object stored in \code{annotatr_cache}. To view a custom annotation, do \code{annotatr_cache$get(name)}. To add a custom annotation to the set of annotations, include \code{'[genome]_custom_[name]'} in the call to \code{build_annotations()}. See example below.
#'
#' @examples
#'
#'  # Read in a BED3 file as a custom annotation
#'  file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
#'  read_annotations(con = file, name = 'test', genome = 'hg19')
#'  build_annotations(genome = 'hg19', annotations = 'hg19_custom_test')
#'
#'  print(annotatr_cache$get('hg19_custom_test'))
#'
#' @export
read_annotations = function(con, name, genome = NA, format, extraCols = character(), ...) {

    if(missing(name)) {
        name = 'annotations'
    }
    if(is.na(genome)) {
        genome_name = 'genome'
    } else {
        genome_name = genome
    }

    protected_extraCols = c('gene_id','symbol','tx_id')

    if(!missing(format)) {
        gr = rtracklayer::import(con = con, genome = genome, format = format, extraCols = extraCols, ...)
    } else {
        gr = rtracklayer::import(con = con, genome = genome, extraCols = extraCols, ...)
    }

    # Determine whether gene_id or symbol are missing from extraCols
    missing_extraCols = base::setdiff(protected_extraCols, names(extraCols))

    if(any(missing_extraCols == 'gene_id')) {
        GenomicRanges::mcols(gr)$gene_id = NA
    }
    if(any(missing_extraCols == 'symbol')) {
        GenomicRanges::mcols(gr)$symbol = NA
    }
    if(any(missing_extraCols == 'tx_id')) {
        GenomicRanges::mcols(gr)$tx_id = NA
    }

    GenomicRanges::mcols(gr)$id = paste0(name,':',seq_along(gr))
    GenomicRanges::mcols(gr)$type = sprintf('%s_custom_%s', genome_name, name)

    # Make sure only the desired mcols make it out
    GenomicRanges::mcols(gr) = GenomicRanges::mcols(gr)[,c('id','tx_id','gene_id','symbol','type')]

    ########################################################
    # Write the object named [genome]_custom_[name] to the annotatr_cache
    annotatr_cache$set(sprintf('%s_custom_%s', genome_name, name), gr)
}

context('Test intersect/annotate module')

################################################################################
# Test errors

test_that('Test error thrown for non-GRanges regions object in annotate_regions()',{
    annotations = c('hg19_cpg_islands')

    bed = system.file('extdata', 'test_intersect.bed', package = 'annotatr')
    r = suppressMessages(read_regions(con = bed, format = 'bed'))

    expect_error(
        annotate_regions(
            regions = bed,
            annotations = annotations,
            ignore.strand = TRUE,
            quiet = TRUE),
        "regions object is not GRanges")

    expect_error(
        annotate_regions(
            regions = r,
            annotations = bed,
            ignore.strand = TRUE,
            quiet = TRUE),
        "annotations object is not GRanges")

    a_file = system.file('extdata', 'test_annotation_nooverlap.bed', package = 'annotatr')
    read_annotations(con = a_file, name = 'test')
    annotations = build_annotations(genome = 'hg19', annotations = 'genome_custom_test')
    expect_error(
        annotate_regions(
            regions = r,
            annotations = annotations,
            ignore.strand = TRUE,
            quiet = TRUE),
        "No annotations intersect the regions")
})

################################################################################
# Test annotate_regions()

test_that('Test a la carte annotations in annotate_regions()',{
    # Get premade CpG annotations
    annots = expand_annotations('hg19_cpgs')
    data('annotations', package = 'annotatr')

    bed = system.file('extdata', 'test_intersect.bed', package = 'annotatr')
    r = read_regions(con = bed, format = 'bed')

    i = annotate_regions(
        regions = r,
        annotations = annotations,
        ignore.strand = TRUE,
        quiet = TRUE)

    expect_true( all(unique(i$annot$type) %in% expand_annotations(annots)) )
})

test_that('Test a la carte and shortcut annotations in annotate_regions()',{
    data('annotations', package = 'annotatr')

    file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    i = annotate_regions(
        regions = r,
        annotations = annotations,
        ignore.strand = TRUE,
        quiet = TRUE)

    expect_true( all(unique(i$annot$type) %in% c('hg19_cpg_islands', 'hg19_cpg_shores', 'hg19_cpg_shelves', 'hg19_cpg_inter')) )
})

test_that('Custom annotations work in annotate_regions()', {
    r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
    extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
    r = read_regions(con = r_file, extraCols = extraCols, rename_score = 'coverage')

    a_file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
    read_annotations(con = a_file, name = 'TFBS', genome = 'hg19')

    annots = c('hg19_custom_TFBS', 'hg19_cpgs')
    annotations = build_annotations(genome = 'hg19', annotations = annots)

    a = annotate_regions(
        regions = r,
        annotations = annotations,
        ignore.strand = TRUE,
        quiet = TRUE)

    expect_equal(length(a) == 10, expected = TRUE)
})

test_that('annotate_regions() works with only custom annotations', {
    r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
    extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
    r = read_regions(con = r_file, extraCols = extraCols, rename_score = 'coverage')

    a_file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
    read_annotations(con = a_file, name = 'TFBS')
    annotations = build_annotations(genome = 'hg19', annotations = 'genome_custom_TFBS')

    a = annotate_regions(
        regions = r,
        annotations = annotations,
        ignore.strand = TRUE,
        quiet = FALSE)

    expect_equal(length(a) == 5, expected = TRUE)
})

test_that('annotate_regions() uses minoverlap correctly', {
    file = system.file('extdata', 'test_BED3.bed', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    a_file = system.file('extdata', 'test_annotations_minoverlap.bed', package='annotatr')
    read_annotations(con = a_file, name = 'TFBS')
    annotations = build_annotations(genome = 'hg19', annotations = 'genome_custom_TFBS')

    a = annotate_regions(
        regions = r,
        annotations = annotations,
        minoverlap = 5)

    expect_equal(length(a) == 2, expected = TRUE)
    expect_true(all(GenomicRanges::start(a) == c(10791,28801)))
})

context('Test read module')

################################################################################
# Test warnings in read_regions()

test_that('Test rename_* warnings', {
    file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')

    expect_warning(
        read_regions(con = file, format = 'bed', rename_name = 'hello'),
        'Ignoring rename_name parameter because')

    expect_warning(
        read_regions(con = file, format = 'bed', rename_score = 'score'),
        'Ignoring rename_score parameter because')
})

################################################################################
# Test BED3-6+ and bedGraph

test_that('Test BED3', {
    file = system.file('extdata', 'test_BED3.bed', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    expect_true(is(r, 'GRanges'))
})

test_that('Test BED4', {
    file = system.file('extdata', 'test_BED4.bed', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    expect_true(is(r, 'GRanges'))
})

test_that('Test BED5', {
    file = system.file('extdata', 'test_BED5.bed', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    expect_true(is(r, 'GRanges'))
})

test_that('Test BED6', {
    file = system.file('extdata', 'test_BED6.bed', package = 'annotatr')
    r = read_regions(con = file, format = 'bed')

    expect_true(is(r, 'GRanges'))
})

test_that('Test BED6+ with renaming', {
    file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
    extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
    r = read_regions(con = file, extraCols = extraCols, rename_score = 'pval', rename_name = 'DM_status', format = 'bed')

    expect_true(is(r, 'GRanges'))
})

test_that('Test bedGraph', {
    file = system.file('extdata', 'test_bedGraph.bedGraph', package = 'annotatr')
    r = read_regions(con = file, format = 'bedGraph')

    expect_true(is(r, 'GRanges'))
})

################################################################################
# Test

test_that('Test custom BED3 with no genome and a name', {
    file = system.file('extdata', 'test_annotations_3.bed', package = 'annotatr')
    read_annotations(con = file, name = 'test', format = 'bed')

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_test'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED3 with no name and a genome', {
    file = system.file('extdata', 'test_annotations_3.bed', package = 'annotatr')
    read_annotations(con = file, genome = 'hg19', format = 'bed')

    expect_true( all(colnames(mcols(annotatr_cache$get('hg19_custom_annotations'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED3 with no name or genome', {
    file = system.file('extdata', 'test_annotations_3.bed', package = 'annotatr')
    read_annotations(con = file, format = 'bed')
    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_annotations'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED4', {
    file = system.file('extdata', 'test_annotations_4.bed', package = 'annotatr')
    read_annotations(con = file, format = 'bed')

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_test'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED5', {
    file = system.file('extdata', 'test_annotations_5.bed', package = 'annotatr')
    read_annotations(con = file, format = 'bed')

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_test'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED6', {
    file = system.file('extdata', 'test_annotations_6.bed', package = 'annotatr')
    read_annotations(con = file, name = 'six', format = 'bed')

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_six'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED6 with gene_id', {
    file = system.file('extdata', 'test_annotations_6_gene.bed', package = 'annotatr')
    extraCols = c(gene_id = 'character')
    read_annotations(con = file, name = 'geneid', format = 'bed', extraCols = extraCols)

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_geneid'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED6 with symbol', {
    file = system.file('extdata', 'test_annotations_6_symbol.bed', package = 'annotatr')
    extraCols = c(symbol = 'character')
    read_annotations(con = file, name = 'symbol', format = 'bed', extraCols = extraCols)

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_symbol'))) == c('id','tx_id','gene_id','symbol','type')) )
})

test_that('Test custom BED6 with symbol nad gene_id', {
    file = system.file('extdata', 'test_annotations_6_tx_gene_symbol.bed', package = 'annotatr')
    extraCols = c(gene_id = 'character', symbol = 'character', tx_id = 'character')
    read_annotations(con = file, name = 'txgenesymbol', format = 'bed', extraCols = extraCols)

    expect_true( all(colnames(mcols(annotatr_cache$get('genome_custom_txgenesymbol'))) == c('id','tx_id','gene_id','symbol','type')) )
})

context('Test randomize module')

################################################################################
# Setup objects for plot_categorical()

    file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
    regions_genome = read_regions(con = file, genome = 'hg19', format = 'bed')
    regions_nogenome = read_regions(con = file, format = 'bed')

################################################################################
# Test errors

    test_that('Test errors', {
        expect_error(
            randomize_regions(regions = 'hello', allow.overlaps = TRUE, per.chromosome = TRUE),
            'regions must have class GRanges')
        expect_error(
            randomize_regions(regions = regions_nogenome),
            'GRanges object must have a valid genome'
            )
    })

################################################################################
# Test randomize_regions()

    test_that('Test randomized regions', {
        random_regions = randomize_regions(
            regions = regions_genome,
            allow.overlaps = TRUE,
            per.chromosome = TRUE)

        expect_equal(class(random_regions)[1], expected = 'GRanges')
        expect_equal(length(random_regions), expected = length(regions_genome))
    })

#' Randomize Regions
#'
#' \code{randomize_regions} is a wrapper function for \code{regioneR::randomizeRegions()} that simplifies the creation of randomized regions for an input set of regions read with \code{read_regions()}. It relies on the \code{seqlengths} of \code{regions} in order to build the appropriate \code{genome} object for \code{regioneR::randomizeRegions()}.
#'
#' NOTE: The data associated with the input \code{regions} are not passed on to the random regions.
#'
#' @param regions A \code{GRanges} object from \code{read_regions}.
#' @param allow.overlaps A logical stating whether random regions can overlap input regions (TRUE) or not (FALSE). Default TRUE.
#' @param per.chromosome A logical stating whether the random regions should remain on the same chromosome (TRUE) or not (FALSE). Default TRUE.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{GRanges} object of randomized regions based on \code{regions} from \code{read_regions()}. NOTE: Data associated with the original regions is not attached to the randomized regions.
#'
#' @examples
#'    # Create random region set based on ENCODE ChIP-seq data
#'    file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
#'    r = read_regions(con = file, genome = 'hg19')
#'
#'    random_r = randomize_regions(regions = r)
#'
#' @export
randomize_regions = function(regions, allow.overlaps = TRUE, per.chromosome = TRUE, quiet = FALSE) {

    ########################################################################
    # Argument parsing and error handling
    if(class(regions)[1] != "GRanges") {
        stop('Error: regions must have class GRanges. The best way to ensure this is to pass the result of read_regions() into this function.')
    }

    # Get the genome from the regions
    genome = unique(Seqinfo::genome(regions))

    if(is.na(genome)) {
        stop('Error: regions GRanges object must have a valid genome to randomize its regions.')
    } else {
        chr_lengths = Seqinfo::Seqinfo(genome = genome)
        chr_lengths = Seqinfo::seqlengths(chr_lengths)

        df_genome = data.frame(
            'chr' = names(chr_lengths),
            'start' = rep.int(1, length(chr_lengths)),
            'end' = as.numeric(chr_lengths),
            stringsAsFactors = FALSE)
    }

    if(!quiet) {
        message('Randomizing regions...')
    }

    # Randomize the regions
    randomized = regioneR::randomizeRegions(A = regions, genome = df_genome,
        per.chromosome = per.chromosome, allow.overlaps = allow.overlaps)

    # Sort the randomized
    randomized = sort(randomized)

    return(randomized)
}

#' Summarize annotation counts
#'
#' Given a \code{GRanges} of annotated regions, count the number of regions in each annotation type. If \code{annotated_random} is not \code{NULL}, then the same is computed for the random regions.
#'
#' If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once. For example, if a region were annotated to multiple exons, it would only count once toward the exons, but if it were annotated to an exon and an intron, it would count towards both.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param annotated_random The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{tbl_df} of the number of regions per annotation type.
#'
#' @examples
#'    ### An example of ChIP-seq peaks with signalValue
#'
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
#'    r = read_regions(con = file, genome = 'hg19')
#'
#'    a = annotate_regions(
#'        regions = r,
#'        annotations = annotations,
#'        ignore.strand = TRUE,
#'        quiet = FALSE)
#'
#'    rnd = randomize_regions(regions = r)
#'
#'    rnd_annots = annotate_regions(
#'        regions = rnd,
#'        annotations = annotations,
#'        ignore.strand = TRUE,
#'        quiet = FALSE)
#'
#'    # Summarize the annotated regions without randomized regions
#'    s = summarize_annotations(annotated_regions = a)
#'
#'    # Summarize the annotated regions with randomized regions
#'    s_rnd = summarize_annotations(
#'        annotated_regions = a,
#'        annotated_random = rnd_annots)
#'
#' @export
summarize_annotations = function(annotated_regions, annotated_random, quiet = FALSE) {
    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # If a region has multiple annotation types that are the same, count only one
    # from each type of annotation
    annotated_regions = dplyr::distinct(
        dplyr::ungroup(annotated_regions),
        across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)

    # Tally over data and random regions if annotated_random isn't null,
    # otherwise tally over data only
    if(!missing(annotated_random)) {
        # Tidy the GRanges into a tbl_df for use with dplyr functions
        annotated_random = as.data.frame(annotated_random, row.names = NULL)

        # If a region has multiple annotation types that are the same, count only one
        # from each type of annotation
        annotated_random = dplyr::distinct(
            dplyr::ungroup(annotated_random),
            across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)

        if(!quiet) {
            message('Counting annotation types in data and random regions')
        }

        combined_annots = dplyr::bind_rows('Data' = annotated_regions, 'Random Regions' = annotated_random, .id = 'data_type')

        agg = dplyr::tally(
            dplyr::group_by(combined_annots, across(c('data_type', 'annot.type')))
        )
    } else {
        if(!quiet) {
            message('Counting annotation types')
        }

        # Tally over the normal data
        agg = dplyr::tally(
            dplyr::group_by(annotated_regions, across(c('annot.type')))
        )
    }

    return(agg)
}

#' Summarize numerical data over groupings of annotated regions
#'
#' Given a \code{GRanges} of annotated regions, summarize numerical data columns based on a grouping.
#'
#' NOTE: We do not take the distinct values of \code{seqnames}, \code{start}, \code{end}, \code{annot.type} as in the other \code{summarize_*()} functions because in the case of a region that intersected two distinct exons, using \code{distinct()} would destroy the information of the mean of the numerical column over one of the exons, which is not desirable.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param by A character vector of the columns of \code{as.data.frame(annotated_regions)} to group over. Default is \code{c(annot.type, annot.id)}.
#' @param over A character vector of the numerical columns in \code{as.data.frame(annotated_regions)} to \code{count}, take the \code{mean}, and take the \code{sd} over after grouping according to the \code{by} column. NOTE: If more than one value is used, the naming scheme for the resuling \code{dplyr::tbl} summary columns are \code{COLNAME_n}, \code{COLNAME_mean}, \code{COLNAME_sd}. If \code{over} has length one, then the column names are \code{n}, \code{mean}, \code{sd}.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A grouped \code{dplyr::tbl_df}, and the \code{count}, \code{mean}, and \code{sd} of the \code{cols} \code{by} the groupings.
#'
#' @examples
#' ### Test on a very simple bed file to demonstrate different options
#'
#' # Get premade CpG annotations
#' data('annotations', package = 'annotatr')
#'
#' r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
#' extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
#' r = read_regions(con = r_file, genome = 'hg19', extraCols = extraCols, rename_score = 'coverage')
#'
#' a = annotate_regions(
#'        regions = r,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#' # Testing over normal by
#' sn1 = summarize_numerical(
#'        annotated_regions = a,
#'        by = c('annot.type', 'annot.id'),
#'        over = c('coverage', 'mu1', 'mu0'),
#'        quiet = FALSE)
#'
#' # Testing over a different by
#' sn2 = summarize_numerical(
#'        annotated_regions = a,
#'        by = c('diff_exp'),
#'        over = c('coverage', 'mu1', 'mu0'))
#'
#' @export
summarize_numerical = function(annotated_regions, by = c('annot.type', 'annot.id'), over, quiet = FALSE) {
    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    if(missing(over)) {
        stop("Error: over cannot be missing.")
    }

    if(!quiet) {
        message(sprintf('Grouping regions by %s, and summarizing numerical data over %s',
            paste(by, collapse=' & '), paste(over, collapse=' & ')))
    }
    agg = dplyr::summarize_at(
        dplyr::group_by(annotated_regions, across(by)),
        over,
        dplyr::funs(n(), 'mean', 'sd'))

    return(agg)
}

#' Summarize categorical data over groupings of annotated regions
#'
#' Given a \code{GRanges} of annotated regions, count the number of regions when the annotations are grouped \code{by} categorical columns.
#'
#' If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once. For example, if a region were annotated to multiple exons, it would only count once toward the exons, but if it were annotated to an exon and an intron, it would count towards both.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param by A character vector to group the data in \code{as.data.frame(annotated_regions)} by and tally over. Default is \code{c('annot.type', 'annot.id')}.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A grouped \code{dplyr::tbl_df} of the counts of groupings according to the \code{by} vector.
#'
#' @examples
#'
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
#'    extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
#'    r = read_regions(con = r_file, genome = 'hg19', extraCols = extraCols, rename_score = 'coverage')
#'
#'    a = annotate_regions(
#'        regions = r,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    sc = summarize_categorical(
#'        annotated_regions = a,
#'        by = c('annot.type', 'name'),
#'        quiet = FALSE)
#'
#' @export
summarize_categorical = function(annotated_regions, by = c('annot.type', 'annot.id'), quiet = FALSE) {
    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # If a region has multiple annotation types that are the same, count only one
    # from each type of annotation
    annotated_regions = dplyr::distinct(
        dplyr::ungroup(annotated_regions),
        across(c('seqnames', 'start', 'end', by)), .keep_all=TRUE)

    if(!quiet) {
        message(sprintf('Grouping regions by %s, and tallying',
            paste(by, collapse=' & ')))
    }

    agg = dplyr::tally(
        dplyr::group_by(annotated_regions, across(by)))

    return(agg)
}

context('Test summarize module')

data('annotations', package = 'annotatr')

bed = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
r = suppressMessages(read_regions(con = bed, genome = 'hg19', extraCols = extraCols, rename_score = 'pval', rename_name = 'DM_status', format = 'bed'))
r = r[1:1000]
r$cancer_status = 'Cancer'
r2 = r
r2$cancer_status = 'NoCancer'

r_dup = c(r,r2)

a = suppressMessages(annotate_regions(
    regions = r,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

a_dup = suppressMessages(annotate_regions(
    regions = r_dup,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

rnd = suppressMessages(randomize_regions(regions = r))

rnd_annot = suppressMessages(annotate_regions(
    regions = rnd,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

################################################################################
# Test errors

test_that('Test for error with over=NULL in summarize_numerical()',{
    expect_error(summarize_numerical(annotated_regions = a),
        'over cannot be missing')
})

################################################################################
# Test summarize functions

test_that('Test summarize_annotations()', {
    s = summarize_annotations(annotated_regions = a, quiet = FALSE)

    srand = summarize_annotations(
        annotated_regions = a,
        annotated_random = rnd_annot,
        quiet = FALSE)

    # NOTE: For small data it is possible that the random regions won't
    # intersect all CpG types so the second test may fail. Moreover,
    # if you are going to compute fold changes, corresponding random
    # rows may be missing if the data is too small...
    expect_equal( sum(s[['n']]), expected = 1064)
    expect_equal( nrow(srand), expected = 8)
})

test_that('Test summarize_numerical()', {
    s = summarize_numerical(
        annotated_regions = a,
        by = c('annot.type', 'annot.id'),
        over = 'diff_meth',
        quiet = TRUE)

    expect_equal( mean(s[['mean']]), expected = 2.424537, tolerance = 0.01)
})

test_that('Test summarize_numerical() and summarize_categorical() over small data', {
    # Testing summarize_numerical()
    sn1 = summarize_numerical(
        annotated_regions = a,
        by = c('annot.type', 'annot.id'),
        over = 'diff_meth',
        quiet = FALSE)
    sn2 = summarize_numerical(
        annotated_regions = a,
        by = c('DM_status'),
        over = c('diff_meth', 'mu1', 'mu0'),
        quiet = TRUE)

    # Testing summarize_categorical()
    sc1 = summarize_categorical(
        annotated_regions = a,
        by = c('annot.type', 'DM_status'),
        quiet = FALSE)

    # Testing maintanence of duplicate regions with different categories
    sc2 = summarize_categorical(
        annotated_regions = a_dup,
        by = c('annot.type', 'cancer_status'),
        quiet = FALSE)

    expect_equal( sn1[['mean']][which(sn1[['annot.id']] == 'inter:8599')], expected = -1.0066888, tolerance = 0.01)
    expect_equal( sn2[['mu0_mean']][which(sn2[['DM_status']] == 'hyper')], expected = 16.34614, tolerance = 0.01)

    expect_equal( sc1[['n']][which(sc1[['annot.type']] == 'hg19_cpg_inter' & sc1[,'DM_status'] == 'hyper')], expected = 19)

    expect_true( sc2[['n']][which(sc2[['annot.type']] == 'hg19_cpg_inter' & sc2[,'cancer_status'] == 'Cancer')] == sc2[['n']][which(sc2[['annot.type']] == 'hg19_cpg_inter' & sc2[,'cancer_status'] == 'NoCancer')] )
})

#' Plot the number of regions per annotation
#'
#' Given a \code{GRanges} of annotated regions, plot the number of regions with the corresponding genomic annotations used in \code{annotation_order}. If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once in the corresponding bar plot. For example, if a region were annotated to multiple exons, it would only count once toward the exon bar in the plot, but if it were annotated to an exon and an intron, it would count towards both.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param annotated_random The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}.
#' @param annotation_order A character vector which doubles as the subset of annotations desired for the plot as well as the ordering. If \code{NULL}, all annotations are displayed.
#' @param plot_title A string used for the title of the plot. If missing, no title is displayed.
#' @param x_label A string used for the x-axis label. If missing, no x-axis label is displayed.
#' @param y_label A string used for the y-axis label. If missing, no y-axis label is displayed.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{ggplot} object which can be viewed by calling it, saved with \code{ggplot2::ggsave}, or edited.
#'
#' @examples
#'    ########################################################################
#'    # An example of ChIP-seq peaks with signalValue used for score
#'
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    chip_bed = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
#'    chip_regions = read_regions(con = chip_bed, genome = 'hg19')
#'
#'    chip_rnd = randomize_regions(regions = chip_regions)
#'
#'    chip_annots = annotate_regions(
#'        regions = chip_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    chip_rnd_annots = annotate_regions(
#'        regions = chip_rnd,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    annots_order = c(
#'        'hg19_cpg_islands',
#'        'hg19_cpg_shores')
#'
#'    p_annots = plot_annotation(annotated_regions = chip_annots,
#'        annotation_order = annots_order)
#'    p_annots_rnd = plot_annotation(annotated_regions = chip_annots,
#'        annotated_random = chip_rnd_annots, annotation_order = annots_order)
#'
#' @export
plot_annotation = function(annotated_regions, annotated_random, annotation_order = NULL,
    plot_title, x_label, y_label, quiet = FALSE) {

    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # Order and subset the annotations
    annotated_regions = subset_order_tbl(tbl = annotated_regions, col='annot.type', col_order=annotation_order)

    ########################################################################
    # If a region has multiple annotation types that are the same, count only one
    # from each type of annotation
    annotated_regions = dplyr::distinct(
        dplyr::ungroup(annotated_regions),
        across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)

    # Do particular things if annotated_random isn't NULL
    if(!missing(annotated_random)) {
        # Tidy the GRanges into a tbl_df for use with dplyr functions
        annotated_random = as.data.frame(annotated_random, row.names = NULL)

        # Order and subset the randomized annotations
        annotated_random = subset_order_tbl(tbl = annotated_random, col='annot.type', col_order=annotation_order)

        # If a region has multiple annotation types that are the same, count only one
        # from each type of annotation
        annotated_random = dplyr::distinct(
            dplyr::ungroup(annotated_random),
            across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)

        # Combine the tbl_dfs in preparation for visualization
        annotated_regions = dplyr::bind_rows("Data" = annotated_regions, "Random Regions" = annotated_random, .id = 'data_type')
    }

    ########################################################################
    # Construct the plot

    # Make the base ggplot
    # NOTE: binwidth may need to be a parameter
    if(missing(annotated_random)) {
        plot =
        ggplot(annotated_regions, aes(x = .data[["annot.type"]])) +
            geom_bar() +
            theme_bw() +
            theme(axis.text.x = element_text(angle = 30, hjust = 1),
                legend.title=element_blank(), legend.position="bottom", legend.key = element_rect(color = 'white'))
    } else {
        plot =
            ggplot(annotated_regions, aes(x = .data[["annot.type"]])) +
            geom_bar(aes(fill = .data[["data_type"]]), position='dodge') +
            theme_bw() +
            scale_fill_grey() +
            theme(axis.text.x = element_text(angle = 30, hjust = 1),
                legend.title=element_blank(), legend.position="bottom", legend.key = element_rect(color = 'white'))
    }

    # Add any user defined labels to the plot if their values are not NULL
    # if they are NULL, ggplot() will use defaults
    if(!missing(plot_title)) {
        plot = plot + ggtitle(plot_title)
    }
    if(!missing(x_label)) {
        plot = plot + xlab(x_label)
    }
    if(!missing(y_label)) {
        plot = plot + ylab(y_label)
    }

    return(plot)
}

#' Plot pair-wise annotations across regions
#'
#' All co-occurring annotations associated with a region are computed and displayed as a heatmap.
#'
#' As with \code{plot_annotation()}, the number in each cell is the number of unique regions annotated to the pair of annotations.
#'
#' For example, if a region is annotated to both a CpG shore and to two different exons simultaneously, the region will only be counted once in the CpG shore / exon cell. NOTE, this same region will count once in both the CpG shore and exon cells on the diagonal.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param annotation_order A character vector which doubles as the subset of annotations desired for plot as well as the ordering. If \code{NULL}, all annotations are displayed.
#' @param plot_title A string used for the title of the plot. If missing, no plot title label is displayed.
#' @param axes_label A string used for the axis labels. If missing, corresponding variable name used.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{ggplot} object which can be viewed by calling it, saved with \code{ggplot2::ggsave}, or edited.
#'
#' @examples
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
#'    extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
#'    dm_regions = read_regions(con = dm_file, extraCols = extraCols,
#'        rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
#'    dm_regions = dm_regions[1:1000]
#'
#'    dm_annots = annotate_regions(
#'        regions = dm_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    all_order = c(
#'        'hg19_cpg_islands',
#'        'hg19_cpg_shores',
#'        'hg19_cpg_shelves',
#'        'hg19_cpg_inter')
#'
#'    dm_vs_ca = plot_coannotations(
#'        annotated_regions = dm_annots,
#'        annotation_order = all_order,
#'        axes_label = 'Annotations',
#'        plot_title = 'Co-occurrence of Annotations')
#'
#' @export
plot_coannotations = function(annotated_regions, annotation_order = NULL,
    plot_title, axes_label, quiet = FALSE) {

    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # Order and subset the annotations
    annotated_regions = subset_order_tbl(tbl = annotated_regions, col='annot.type', col_order=annotation_order)

    ########################################################################
    # Find the co-annotations

    annotation_pairs_by_region = dplyr::do(
        dplyr::group_by(annotated_regions, across(c('seqnames', 'start', 'end'))),
        expand.grid(.$annot.type, .$annot.type, stringsAsFactors = FALSE))

    annotation_pairs_by_region = dplyr::distinct(dplyr::ungroup(annotation_pairs_by_region),
        across(c('seqnames', 'start', 'end', 'Var1', 'Var2')), .keep_all=TRUE)

    pairwise_annotation_counts = table(annotation_pairs_by_region[['Var1']], annotation_pairs_by_region[['Var2']])

    pac_m = reshape2::melt(pairwise_annotation_counts, value.name = 'Counts')

    ########################################################################
    # Construct the plot

    # Make the base ggplot
    # NOTE: binwidth may need to be a parameter
    plot = ggplot(pac_m, aes(x = .data[["Var1"]], y = .data[["Var2"]])) +
        geom_raster(aes(fill = .data[["Counts"]])) +
        geom_text(aes(label = .data[["Counts"]])) +
        scale_fill_gradient(low = "white", high = "steelblue") +
        theme(axis.text.x = element_text(angle = 30, hjust = 1), axis.text.y = element_text(angle = 30, hjust = 1))

    # Add any user defined labels to the plot if their values are not NULL
    # if they are NULL, ggplot() will use defaults
    if(!missing(plot_title)) {
        plot = plot + ggtitle(plot_title)
    }
    if(!missing(axes_label)) {
        plot = plot + xlab(axes_label)
        plot = plot + ylab(axes_label)
    }

    return(plot)
}

#' Plot numerical data over regions or regions summarized over annotations
#'
#' This function produces either histograms over \code{facet}, or x-y scatterplots over \code{facet}. In the case of histograms over facets, the All distribution (hollow histogram with red outline) is the distribution of \code{x} over all the regions in the data. The facet specific distributions (solid gray) are the distribution of \code{x} over the regions in each facet. For example, a CpG with associated percent methylation annotated to a CpG island and a promoter will count once in the All distribution, but will count once each in the CpG island and promoter facet distributions.
#'
#' @param annotated_regions A \code{GRanges} returned from \code{annotate_regions()}. If the data is not summarized, the data is at the region level. If it is summarized, it represents the average or standard deviation of the regions by the character vector used for \code{by} in \code{summarize_numerical()}.
#' @param x A string indicating the column of the \code{GRanges} to use for the x-axis.
#' @param y A string indicating the column of the \code{GRanges} to use for the y-axis. If missing, a a histogram over \code{x} will be plotted. If not missing, a scatterplot is plotted.
#' @param facet A string, or character vector of two strings, indicating indicating which categorical variable(s) in the \code{GRanges} to make \code{ggplot2} facets over. When two facets are given, the first entry is the vertical facet and the second entry is the horizontal facet. Default is \code{annot.type}.
#' @param facet_order A character vector, or list of character vectors if \code{facet} has length 2, which gives the order of the facets, and can be used to subset the column in the \code{GRanges} used for the \code{facet}. For example, if \code{facet = 'annot.type'}, then the annotations maybe subsetted to just CpG annotations. Default is \code{NULL}, meaning all annotations in their default order are used.
#' @param bin_width An integer indicating the bin width of the histogram used for score. Default 10. Select something appropriate for the data. NOTE: This is only used if \code{y} is \code{NULL}.
#' @param plot_title A string used for the title of the plot. If missing, no title is displayed.
#' @param x_label A string used for the x-axis label. If missing, no x-axis label is displayed.
#' @param y_label A string used for the y-axis label. If missing, no y-axis label is displayed.
#' @param legend_facet_label A string used to label the gray bar portion of the legend. Defaults to "x in facet".
#' @param legend_cum_label A string used to label the red outline portion of the legend. Defaults to "All in x".
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
#'
#' @examples
#'    # An example with multi-columned data
#'
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
#'    extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
#'    dm_regions = read_regions(con = dm_file, extraCols = extraCols,
#'        rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
#'    dm_regions = dm_regions[1:1000]
#'
#'    # Annotate the regions
#'    dm_annots = annotate_regions(
#'        regions = dm_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    # Plot histogram of group 1 methylation rates across the CpG annotations.
#'    # NOTE: Overall distribution (everything in \code{facet_order})
#'    # is plotted in each facet for comparison.
#'    dm_vs_regions_mu1 = plot_numerical(
#'        annotated_regions = dm_annots,
#'        x = 'mu1',
#'        facet = 'annot.type',
#'        facet_order = c('hg19_cpg_islands','hg19_cpg_shores',
#'            'hg19_cpg_shelves','hg19_cpg_inter'),
#'        bin_width = 5,
#'        plot_title = 'Group 1 Methylation over CpG Annotations',
#'        x_label = 'Group 1 Methylation')
#'
#'    # Plot histogram of group 1 methylation rates across the CpG annotations
#'    # crossed with DM_status
#'    dm_vs_regions_diffmeth = plot_numerical(
#'        annotated_regions = dm_annots,
#'        x = 'diff_meth',
#'        facet = c('annot.type','DM_status'),
#'        facet_order = list(
#'            c('hg19_genes_promoters','hg19_genes_5UTRs','hg19_cpg_islands'),
#'            c('hyper','hypo','none')),
#'        bin_width = 5,
#'        plot_title = 'Group 0 Region Methylation In Genes',
#'        x_label = 'Methylation Difference')
#'
#'    # Can also use the result of annotate_regions() to plot two numerical
#'    # data columns against each other for each region, and facet by annotations.
#'    dm_vs_regions_annot = plot_numerical(
#'        annotated_regions = dm_annots,
#'        x = 'mu0',
#'        y = 'mu1',
#'        facet = 'annot.type',
#'        facet_order = c('hg19_cpg_islands','hg19_cpg_shores',
#'            'hg19_cpg_shelves','hg19_cpg_inter'),
#'        plot_title = 'Region Methylation: Group 0 vs Group 1',
#'        x_label = 'Group 0',
#'        y_label = 'Group 1')
#'
#'    # Another example, but using differential methylation status as the facets.
#'    dm_vs_regions_name = plot_numerical(
#'        annotated_regions = dm_annots,
#'        x = 'mu0',
#'        y = 'mu1',
#'        facet = 'DM_status',
#'        facet_order = c('hyper','hypo','none'),
#'        plot_title = 'Region Methylation: Group 0 vs Group 1',
#'        x_label = 'Group 0',
#'        y_label = 'Group 1')
#'
#' @export
plot_numerical = function(annotated_regions, x, y, facet, facet_order, bin_width=10,
    plot_title, x_label, y_label, legend_facet_label, legend_cum_label, quiet = FALSE) {

    # Check for facet facet_order mismatches
    if(length(facet) == 2) {
        if(!is(facet_order, 'list')) {
            stop('When facet is of length two, facet_order must be a list giving the order for each facet variable.')
        }
        two_facets = TRUE
    } else {
        two_facets = FALSE
    }

    # Deal with facet formula
    if(two_facets) {
        facet_formula = paste(facet[1], "~", facet[2])
    } else {
        facet_formula = paste("~", facet)
    }

    # Tidy the GRanges into a tbl_df for use with dplyr functions
    tbl = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # Order and subset the annotations
    if(two_facets) {
        sub_tbl = subset_order_tbl(tbl = tbl, col = facet[1], col_order = facet_order[[1]])
        sub_tbl = subset_order_tbl(tbl = sub_tbl, col = facet[2], col_order = facet_order[[2]])
    } else {
        sub_tbl = subset_order_tbl(tbl = tbl, col = facet, col_order = facet_order)
    }

    ########################################################################
    # Create data objects for plots
    facet_data = dplyr::distinct(dplyr::ungroup(sub_tbl), across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)
    if(two_facets) {
        all_data = dplyr::distinct(dplyr::select(dplyr::ungroup(tbl), -matches(facet[1])), across(c('seqnames', 'start', 'end')), .keep_all=TRUE)
        all_data = dplyr::distinct(dplyr::select(all_data, -matches(facet[2])), across(c('seqnames', 'start', 'end')), .keep_all=TRUE)
    } else {
        all_data = dplyr::distinct(dplyr::select(dplyr::ungroup(tbl), -matches(facet)), across(c('seqnames', 'start', 'end')), .keep_all=TRUE)
    }


    ########################################################################
    # Construct the plot
    # Note, data must be dplyr::ungroup()-ed before hand for the proper
    # display of the overall distribution.

    if(missing(y)) {
        if(missing(legend_facet_label)) {
            if(two_facets) {
                legend_facet_label = sprintf('%s in %s x %s', x, facet[1], facet[2])
            } else {
                legend_facet_label = sprintf('%s in %s', x, facet)
            }
        }
        if(missing(legend_cum_label)) {
            legend_cum_label = sprintf('All %s', x)
        }
        fill_man = c(NA, 'gray')
        names(fill_man) = c(legend_cum_label, legend_facet_label)

        # Make the base histogram ggplot
        plot =
            # Facet hists are plotted with distinct (seqnames, start, end, annot.type) combinations
            ggplot(
                data = facet_data,
                aes(x = .data[[x]], y = after_stat(density))) +
            geom_histogram(binwidth=bin_width, aes(fill = legend_facet_label)) +
            facet_wrap( stats::as.formula(facet_formula) ) + # Over the facets
            # All hist is plotted with distinct (seqnames, start, end) combinations
            geom_histogram(
                data = all_data,
                binwidth=bin_width, aes(fill = legend_cum_label, color = 'red')) + # All the data
            theme_bw() +
            scale_fill_manual(values = fill_man) +
            guides(color = 'none') +
            theme(legend.title=element_blank(), legend.position="bottom", legend.key = element_rect(color = c('red','white')))
    } else {
        # Make the base scatter ggplot
        plot = ggplot(facet_data, aes(x = .data[[x]], y = .data[[y]])) +
            geom_point(alpha = 1/8, size = 1) +
            facet_wrap( stats::as.formula(facet_formula) ) +
            theme_bw()
    }

    # Add any user defined labels to the plot if their values are not NULL
    # if they are NULL, ggplot() will use defaults
    if(!missing(plot_title)) {
        plot = plot + ggtitle(plot_title)
    }
    if(!missing(x_label)) {
        plot = plot + xlab(x_label)
    }
    if(!missing(y_label)) {
        plot = plot + ylab(y_label)
    }

    return(plot)
}

#' Plot numerical data occurring in pairs of annotations
#'
#' Plot numerical data associated with regions occurring in \code{annot1}, \code{annot2} and in both. As with \code{plot_numerical()}, the result is a plot of histograms or x-y scatterplots.
#'
#' For example, a CpG with associated percent methylation annotated to a CpG island and a promoter will count once in the All distribution and once in the CpG island / promoter facet distribution. However, a CpG associated only with a promoter will count once in the All distribution and once in the promoter / promoter distribution.
#'
#' @param annotated_regions A \code{GRanges} returned from \code{annotate_regions()}.
#' @param x A string indicating the column of the \code{GRanges} to use for the x-axis.
#' @param y A string indicating the column of the \code{GRanges} to use for the y-axis. If missing, a histogram over \code{x} will be plotted. If not missing, a scatterplot is plotted.
#' @param annot1 A string indicating the first annotation type.
#' @param annot2 A string indicating the second annotation type.
#' @param bin_width An integer indicating the bin width of the histogram used for score. Default 10. Select something appropriate for the data. NOTE: This is only used if \code{y} is \code{NULL}.
#' @param plot_title A string used for the title of the plot. If missing, no title is displayed.
#' @param x_label A string used for the x-axis label. If missing, no x-axis label is displayed.
#' @param y_label A string used for the y-axis label. If missing, no y-axis label is displayed.
#' @param legend_facet_label A string used to label the gray bar portion of the legend. Defaults to "x in annot pair".
#' @param legend_cum_label A string used to label the red outline portion of the legend. Defaults to "All x".
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
#'
#' @examples
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
#'    extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
#'    dm_regions = read_regions(con = dm_file, extraCols = extraCols,
#'        rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
#'    dm_regions = dm_regions[1:1000]
#'
#'    dm_annots = annotate_regions(
#'        regions = dm_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    dm_vs_num_co = plot_numerical_coannotations(
#'        annotated_regions = dm_annots,
#'        x = 'mu0',
#'        annot1 = 'hg19_cpg_islands',
#'        annot2 = 'hg19_cpg_shelves',
#'        bin_width = 5,
#'        plot_title = 'Group 0 Perc. Meth. in CpG Islands and Promoters',
#'        x_label = 'Percent Methylation')
#'
#' @export
plot_numerical_coannotations = function(annotated_regions, x, y, annot1, annot2, bin_width=10,
    plot_title, x_label, y_label, legend_facet_label, legend_cum_label, quiet = FALSE) {

    # Tidy the GRanges into a tbl_df for use with dplyr functions
    tbl = as.data.frame(annotated_regions, row.names = NULL)

    ########################################################################
    # Order and subset the annotations
    annotation_order = c(annot1,annot2)
    sub_tbl = subset_order_tbl(tbl = tbl, col='annot.type', col_order=annotation_order)

    ########################################################################
    # Find the co-annotations

    # Use combn instead of expand.grid because we do not want regions annotated to
    # a CpG island and a promoter having their data value count in the island / island
    # facet as well as the promoter / promoter facet. We want it *ONLY* in the
    # island / promoter facet. Note, sorting ensures island / promoter and promoter / island
    # are aggregated
    pairs_by_region = dplyr::do(
        dplyr::group_by(sub_tbl, across(c('seqnames', 'start', 'end'))),
        if(nrow(.) == 1) {
            as.data.frame(
                t(
                    utils::combn(
                        rep.int(as.character(.$annot.type), 2)
                    , 2))
                , stringsAsFactors = FALSE)
        } else {
            as.data.frame(
                t(
                    utils::combn(
                        sort(as.character(.$annot.type))
                    , 2))
            , stringsAsFactors = FALSE)
        }
    )

    # Join on the data chromosome locations
    pairs_by_region = dplyr::inner_join(x = pairs_by_region, y = sub_tbl, by = c('seqnames','start','end'))

    ########################################################################
    # Create data objects for plots
    facet_data = dplyr::distinct(dplyr::ungroup(pairs_by_region),
        across(c('seqnames', 'start', 'end', 'V1', 'V2')), .keep_all=TRUE)
    all_data = dplyr::distinct(dplyr::ungroup(tbl), across(c('seqnames', 'start', 'end')), .keep_all=TRUE)

    ########################################################################
    # Construct the plot
    # Note, data must be dplyr::ungroup()-ed before hand for the proper
    # display of the overall distribution.

    if(missing(y)) {
        if(missing(legend_facet_label)) {
            legend_facet_label = sprintf('%s in %s', x, 'annot pair')
        }
        if(missing(legend_cum_label)) {
            legend_cum_label = sprintf('All %s', x)
        }
        fill_man = c(NA, 'gray')
        names(fill_man) = c(legend_cum_label, legend_facet_label)

        # Make the base histogram ggplot
        plot =
            # Facet hists are plotted with distinct (seqnames, start, end, annot1, annot2) combinations
            ggplot(
                data = facet_data,
                aes(x = .data[[x]], y = after_stat(density))) +
            geom_histogram(binwidth=bin_width, aes(fill = legend_facet_label)) +
            facet_wrap( V1 ~ V2 ) + # Over the facets
            # All hist is plotted with distinct (seqnames, start, end) combinations
            geom_histogram(
                data = all_data,
                binwidth=bin_width, aes(fill = legend_cum_label, color = 'red')) + # All the data
            theme_bw() +
            scale_fill_manual(values = fill_man) +
            guides(color = 'none') +
            theme(legend.title=element_blank(), legend.position="bottom", legend.key = element_rect(color = c('red','white')))
    } else {
        # Make the base scatter ggplot
        plot = ggplot(pairs_by_region, aes(x = .data[[x]], y = .data[[y]])) +
            geom_point(alpha = 1/8, size = 1) +
            facet_wrap( V1 ~ V2 ) +
            theme_bw()
    }

    # Add any user defined labels to the plot if their values are not NULL
    # if they are NULL, ggplot() will use defaults
    if(!missing(plot_title)) {
        plot = plot + ggtitle(plot_title)
    }
    if(!missing(x_label)) {
        plot = plot + xlab(x_label)
    }
    if(!missing(y_label)) {
        plot = plot + ylab(y_label)
    }

    return(plot)
}

#' Plot a categorical data variable over another
#'
#' Given a \code{GRanges} of annotated regions from \code{annotate_regions()}, visualize the the distribution of categorical data \code{fill} in categorical data \code{x}. A bar representing the distribution of all \code{fill} in \code{x} will be added according to the contents of \code{fill}. This is the distribution over all values of \code{x}. Additionally, when \code{annotated_random} is not missing, a "Random Regions" bar shows the distribution of random regions over \code{fill}.
#'
#' For example, if a differentially methylated region has the categorical label hyper, and is annotated to a promoter, a 5UTR, two exons, and an intron. Each annotation will appear in the All bar once. Likewise for the hyper bar if the differential methylation status is chosen as \code{x} with \code{annot.type} chosen as \code{fill}.
#'
#' @param annotated_regions The \code{GRanges} result of \code{annotate_regions()}.
#' @param annotated_random The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}. Random regions can only be used with \code{fill == 'annot.type'}.
#' @param x One of 'annot.type' or a categorical data column, indicating whether annotation classes or data classes will appear on the x-axis.
#' @param fill One of 'annot.type', a categorical data column, or \code{NULL}, indicating whether annotation classes or data classes will fill the bars. If \code{NULL} then the bars will be the total counts of the x classes.
#' @param x_order A character vector that subsets and orders the x classes. Default \code{NULL}, uses existing values.
#' @param fill_order A character vector that subsets and orders the fill classes. Default \code{NULL}, uses existing values.
#' @param position A string which has the same possible values as in \code{ggplot2::geom_bar(..., position)}, i.e., 'stack', 'fill', 'dodge', etc.
#' @param plot_title A string used for the title of the plot. If missing, no title is displayed.
#' @param legend_title A string used for the legend title to describe fills (if fill is not \code{NULL}). Default displays corresponding variable name.
#' @param x_label A string used for the x-axis label. If missing, corresponding variable name used.
#' @param y_label A string used for the y-axis label. If missing, corresponding variable name used.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
#'
#' @examples
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
#'    extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
#'    dm_regions = read_regions(con = dm_file, extraCols = extraCols, genome = 'hg19',
#'        rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
#'    dm_regions = dm_regions[1:1000]
#'
#'    dm_annots = annotate_regions(
#'        regions = dm_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    dm_order = c(
#'        'hyper',
#'        'hypo')
#'    cpg_order = c(
#'        'hg19_cpg_islands',
#'        'hg19_cpg_shores',
#'        'hg19_cpg_shelves',
#'        'hg19_cpg_inter')
#'
#'    dm_vn = plot_categorical(
#'        annotated_regions = dm_annots,
#'        x = 'DM_status',
#'        fill = 'annot.type',
#'        x_order = dm_order,
#'        fill_order = cpg_order,
#'        position = 'fill',
#'        legend_title = 'knownGene Annotations',
#'        x_label = 'DM status',
#'        y_label = 'Proportion')
#'
#'    # Create randomized regions
#'    dm_rnd_regions = randomize_regions(regions = dm_regions)
#'    dm_rnd_annots = annotate_regions(
#'        regions = dm_rnd_regions,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#'    dm_vn_rnd = plot_categorical(
#'        annotated_regions = dm_annots,
#'        annotated_random = dm_rnd_annots,
#'        x = 'DM_status',
#'        fill = 'annot.type',
#'        x_order = dm_order,
#'        fill_order = cpg_order,
#'        position = 'fill',
#'        legend_title = 'knownGene Annotations',
#'        x_label = 'DM status',
#'        y_label = 'Proportion')
#'
#' @export
plot_categorical = function(annotated_regions, annotated_random, x, fill=NULL, x_order=NULL, fill_order=NULL,
    position = 'stack', plot_title, legend_title, x_label, y_label, quiet = FALSE) {

    ########################################################################
    # Argument parsing and error handling

    # Tidy the GRanges into a tbl_df for use with dplyr functions
    annotated_regions = as.data.frame(annotated_regions, row.names = NULL)

    # Ensure the value of x is a column name in summarized_cats
    if( !(x %in% colnames(annotated_regions)) ) {
        stop('The column name used for x does not exist in annotated_regions.')
    }

    # Ensure the value of fill is a column name in summarized_cats if it isn't NULL
    # Also ensure fill != x
    if( !is.null(fill) ) {
        if( !(fill %in% colnames(annotated_regions)) ) {
            stop('The column name used for fill does not exist in annotated_regions.')
        }
        if( x == fill ) {
            stop('Error: x cannot equal fill')
        }
    }

    # If !is.null(annotated_random), check that fill = 'annot.type'. This is the
    # only situation where random regions can be used, because the data from the
    # original regions is not transferred to the random ones.
    if(!missing(annotated_random) && fill != 'annot.type') {
        stop('Error: Random regions can only be used in plot_categorical() when fill == "annot.type" since data from the original regions are not transferred to the random regions.')
    }

    # Check valid position argument
    if(position != 'stack' && position != 'fill' && position != 'dodge') {
        stop('Error: position must be one of "stack", "fill", or "dodge"')
    }

    ########################################################################
    # Order and subset based on fill_order
    annotated_regions = subset_order_tbl(tbl = annotated_regions, col = fill, col_order = fill_order)

    # Take the distinct annotation types per unique data region
    annotated_regions = dplyr::distinct(dplyr::ungroup(annotated_regions), across(c('seqnames', 'start', 'end', x, fill)), .keep_all=TRUE)

    ########################################################################
    # Order and subset based on x_order
    if(is.null(x_order)) {
        x_order = unique(annotated_regions[[x]])
    }
    sub_annot_regions = subset_order_tbl(tbl = annotated_regions, col = x, col_order = x_order)

    # Do particular things if annotated_random isn't NULL
    if(!missing(annotated_random)) {
        # Tidy the GRanges into a tbl_df for use with dplyr functions
        annotated_random = as.data.frame(annotated_random, row.names = NULL)

        # Order and subset the randomized annotations
        annotated_random = subset_order_tbl(tbl = annotated_random, col=fill, col_order=fill_order)

        # Take the distinct annotation types per unique random data region
        annotated_random = dplyr::distinct(dplyr::ungroup(annotated_random), across(c('seqnames', 'start', 'end', 'annot.type')), .keep_all=TRUE)

        # Combine the tbl_dfs in preparation for visualization
        annotated_regions = dplyr::bind_rows("All" = annotated_regions, "Random Regions" = annotated_random, .id = 'data_type')
    }

    ########################################################################
    # Construct the plot

    # Make base ggplot
    if(!missing(annotated_random)) {
        plot =
            ggplot(annotated_regions, aes(x = .data[["data_type"]])) +
            geom_bar(aes(fill = .data[[fill]]), position=position, width=0.5) + # The All bar
            geom_bar(data = sub_annot_regions, aes(x = .data[[x]], fill = .data[[fill]]), position=position, width=0.5) + # The subsets bars
            theme(axis.text.x = element_text(angle = 30, hjust = 1))
    } else {
        plot =
            ggplot(annotated_regions, aes(x = 'All')) +
            geom_bar(aes(fill = .data[[fill]]), position=position, width=0.5) + # The All bar
            geom_bar(data = sub_annot_regions, aes(x = .data[[x]], fill = .data[[fill]]), position=position, width=0.5) + # The subsets bars
            theme(axis.text.x = element_text(angle = 30, hjust = 1))
    }

    # Change the fill scale and name if legend_title isn't null
    if(!missing(legend_title)) {
        plot = plot + scale_fill_hue(name=legend_title)
    } else {
        plot = plot + scale_fill_hue()
    }

    # Deal with the x-axis labels to make sure the order is correct
    if(!missing(annotated_random)) {
        plot = plot + scale_x_discrete(limits = c('All', x_order, 'Random Regions'))
    } else {
        if(x == 'annot.type') {
            plot = plot + scale_x_discrete(limits = c('All', names(tidy_annotations(x_order))))
        } else {
            plot = plot + scale_x_discrete(limits = c('All', x_order))
        }
    }

    # Add any user defined labels to the plot if their values are not NULL
    # if they are NULL, ggplot() will use defaults
    if(!missing(plot_title)) {
        plot = plot + ggtitle(plot_title)
    }
    if(!missing(x_label)) {
        plot = plot + xlab(x_label)
    }
    if(!missing(y_label)) {
        plot = plot + ylab(y_label)
    }

    return(plot)
}

context('Test plot module')

################################################################################
# Setup annotation objects
data('annotations', package = 'annotatr')

################################################################################
# Setup objects for plot_categorical()

dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
dm_regions = suppressMessages(read_regions(con = dm_file, genome = 'hg19', extraCols = extraCols, rename_score = 'pval', rename_name = 'DM_status', format = 'bed'))
dm_regions = dm_regions[1:1000]
dm_regions$cancer_status = 'Cancer'
dm_regions2 = dm_regions
dm_regions2$cancer_status = 'NoCancer'

duplicate_regions = c(dm_regions, dm_regions2)

dm_random_regions = suppressMessages(randomize_regions(regions = dm_regions))

dm_annots = suppressMessages(annotate_regions(
    regions = dm_regions,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

dm_dup_annots = suppressMessages(annotate_regions(
    regions = duplicate_regions,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

dm_random_annots = suppressMessages(annotate_regions(
    regions = dm_random_regions,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE))

################################################################################
# Setup order vectors and plots that will work

dm_order = c(
    'hyper',
    'hypo',
    'none')
cpgs_order = c(
    'hg19_cpg_islands',
    'hg19_cpg_shores',
    'hg19_cpg_shelves',
    'hg19_cpg_inter')

################################################################################
# Test plot_annotation()

test_that('Test plot_annotation() errors', {
    expect_warning(
        plot_annotation(annotated_regions = dm_annots, annotation_order = c('hypor','hype','')),
        'elements in col_order that are not present')
})

test_that('Test plot_annotation() success', {
    dm_va_min = plot_annotation(annotated_regions = dm_annots)

    dm_va = plot_annotation(
        annotated_regions = dm_annots,
        annotation_order = cpgs_order,
        plot_title = 'Testing plot title',
        x_label = 'Test x-label',
        y_label = 'Test y-label')

    dm_va_rnd = plot_annotation(
        annotated_regions = dm_annots,
        annotated_random = dm_random_annots,
        annotation_order = NULL,
        plot_title = 'Testing dodged bars',
        x_label = 'Annotation Type',
        y_label = 'Count')

    expect_s3_class(dm_va_min, "ggplot")
    expect_s3_class(dm_va, "ggplot")
    expect_s3_class(dm_va_rnd, "ggplot")
})

################################################################################
# Test plot_coannotations()

test_that('Test plot_coannotations() success', {

    dm_vs_ca = plot_coannotations(
        annotated_regions = dm_annots,
        annotation_order = cpgs_order,
        axes_label = 'Annotations',
        plot_title = 'Co-occurrence of Annotations')

    expect_s3_class(dm_vs_ca, "ggplot")
})

################################################################################
# Test plot_numerical()

test_that('Test plot_numerical() success', {

    dm_vs_regions_mu1 = plot_numerical(
        annotated_regions = dm_annots,
        x = 'mu1',
        facet = 'annot.type',
        facet_order = c('hg19_cpg_islands','hg19_cpg_shores','hg19_cpg_shelves','hg19_cpg_inter'),
        bin_width = 5,
        plot_title = 'Group 1 Methylation over CpG Annotations',
        x_label = 'Group 1 Methylation',
        legend_facet_label = 'Group 1 Methylation Rate in Annotation',
        legend_cum_label = 'Overall Group 1 Methylation Rate')

    dm_vs_regions_annot = plot_numerical(
        annotated_regions = dm_annots,
        x = 'mu0',
        y = 'mu1',
        facet = 'annot.type',
        facet_order = c('hg19_cpg_islands','hg19_cpg_shores','hg19_cpg_shelves','hg19_cpg_inter'),
        plot_title = 'Region Methylation: Group 0 vs Group 1',
        x_label = 'Group 0',
        y_label = 'Group 1')

    dm_vs_regions_name = plot_numerical(
        annotated_regions = dm_annots,
        x = 'mu0',
        y = 'mu1',
        facet = 'DM_status',
        facet_order = c('hyper','hypo','none'),
        plot_title = 'Region Methylation: Group 0 vs Group 1',
        x_label = 'Group 0',
        y_label = 'Group 1')

    dm_vs_regions_mu12 = plot_numerical(
        annotated_regions = dm_annots,
        x = 'mu1',
        facet = c('annot.type', 'DM_status'),
        facet_order = list(c('hg19_cpg_islands','hg19_cpg_shores'), c('hyper','hypo','none')),
        plot_title = 'Region Methylation: Group 0 vs Group 1',
        x_label = 'Group 0',
        y_label = 'Group 1')

    dm_vs_regions_name2 = plot_numerical(
        annotated_regions = dm_annots,
        x = 'mu0',
        y = 'mu1',
        facet = c('annot.type', 'DM_status'),
        facet_order = list(NULL, c('hyper','hypo','none')),
        plot_title = 'Region Methylation: Group 0 vs Group 1',
        x_label = 'Group 0',
        y_label = 'Group 1')

    expect_s3_class(dm_vs_regions_mu1, "ggplot")
    expect_s3_class(dm_vs_regions_annot, "ggplot")
    expect_s3_class(dm_vs_regions_name, "ggplot")
})

################################################################################
# Test plot_numerical_coannotations()

test_that('Test plot_numerical_coannotations()', {
  dm_vs_num_co1 = plot_numerical_coannotations(
    annotated_regions = dm_annots,
    x = 'mu0',
    annot1 = 'hg19_cpg_islands',
    annot2 = 'hg19_cpg_shores',
    bin_width = 5,
    plot_title = 'Group 0 Perc. Meth. in CpG Islands and Promoters',
    x_label = 'Percent Methylation',
    legend_facet_label = 'Perc. Methylation in annotation pair',
    legend_cum_label = 'Overall Perc. Methylation')

  dm_vs_num_co2 = plot_numerical_coannotations(
    annotated_regions = dm_annots,
    x = 'mu0',
    y = 'mu1',
    annot1 = 'hg19_cpg_islands',
    annot2 = 'hg19_cpg_shores',
    bin_width = 5,
    plot_title = 'Group 0 Perc. Meth. in CpG Islands and Promoters',
    x_label = 'Percent Methylation',
    y_label = 'Percent Methylation')

    expect_s3_class(dm_vs_num_co1, "ggplot")
    expect_s3_class(dm_vs_num_co2, "ggplot")
})

################################################################################
# Test plot_categorical()

  test_that('Test plot_categorical() errors', {
    expect_error(
        plot_categorical(
            annotated_regions = dm_annots),
        'argument "x" is missing')

    expect_error(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'testing'),
        'column name used for x does not exist in annotated_regions')

    expect_error(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'DM_status',
            fill = 'testing'),
        'column name used for fill does not exist in annotated_regions')

    expect_error(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'DM_status',
            fill = 'DM_status'),
        'x cannot equal fill')

    expect_error(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'DM_status',
            fill = 'annot.type',
            position = 'no'),
        'position must be one of "stack", "fill"')

    expect_warning(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'DM_status',
            fill = 'annot.type',
            x_order = cpgs_order),
        'elements in col_order that are not present')

    expect_warning(
        plot_categorical(
            annotated_regions = dm_annots,
            x = 'DM_status',
            fill = 'annot.type',
            fill_order = dm_order),
        'elements in col_order that are not present')
  })

test_that('Test plot_categorical() error for random regions and non annot fill', {
    expect_error(
        plot_categorical(
            annotated_regions = dm_annots,
            annotated_random = dm_random_annots,
            x = 'annot.type',
            fill = 'DM_status',
            x_order = cpgs_order,
            fill_order = dm_order,
            position = 'fill',
            legend_title = 'Annotations',
            plot_title = 'DM status by CpG Annotation Proportions',
            x_label = 'DM status',
            y_label = 'Proportion'),
        'since data from the original regions are not transferred to the random regions')
    })

test_that('Test plot_categorical() success', {
    dm_vn_min = plot_categorical(
        annotated_regions = dm_annots,
        x = 'annot.type')

    dm_vn = plot_categorical(
        annotated_regions = dm_annots,
        x = 'DM_status',
        fill = 'annot.type',
        x_order = dm_order,
        fill_order = cpgs_order,
        position = 'fill',
        legend_title = 'knownGene Annotations',
        plot_title = 'DM status in knownGene Annots.',
        x_label = 'DM status',
        y_label = 'Proportion')

    dm_vn_rnd = plot_categorical(
        annotated_regions = dm_annots,
        annotated_random = dm_random_annots,
        x = 'DM_status',
        fill = 'annot.type',
        x_order = dm_order,
        fill_order = cpgs_order,
        position = 'fill',
        legend_title = 'Annotations',
        plot_title = 'DM status by CpG Annotation Proportions',
        x_label = 'DM status',
        y_label = 'Proportion')

    expect_s3_class(dm_vn_min, "ggplot")
    expect_s3_class(dm_vn, "ggplot")
    expect_s3_class(dm_vn_rnd, "ggplot")
})

### Constants
# TxDb.* family of packages
TXDBS = c(
    'TxDb.Dmelanogaster.UCSC.dm3.ensGene',
    'TxDb.Dmelanogaster.UCSC.dm6.ensGene',
    'TxDb.Drerio.UCSC.danRer10.refGene',
    'TxDb.Drerio.UCSC.danRer11.refGene',
    'TxDb.Ggallus.UCSC.galGal5.refGene',
    'TxDb.Hsapiens.UCSC.hg19.knownGene',
    'TxDb.Hsapiens.UCSC.hg38.knownGene',
    'TxDb.Mmusculus.UCSC.mm9.knownGene',
    'TxDb.Mmusculus.UCSC.mm10.knownGene',
    'TxDb.Mmusculus.UCSC.mm39.knownGene',
    'TxDb.Rnorvegicus.UCSC.rn4.ensGene',
    'TxDb.Rnorvegicus.UCSC.rn5.refGene',
    'TxDb.Rnorvegicus.UCSC.rn6.refGene',
    'TxDb.Rnorvegicus.UCSC.rn7.refGene')

# org.* family of packages
ORGDBS = data.frame(
    genome = c('dm3','dm6','danRer10','danRer11','galGal5','hg19','hg38','mm9','mm10','mm39','rn4','rn5','rn6','rn7'),
    org = c('Dm','Dm','Dr','Dr','Gg','Hs','Hs','Mm','Mm','Mm','Rn','Rn','Rn','Rn'),
    stringsAsFactors = FALSE)

HMMCELLLINES = c('Gm12878','H1hesc','Hepg2','Hmec','Hsmm','Huvec','K562','Nhek','Nhlf')

HMMCODES = c('1_Active_Promoter', '2_Weak_Promoter' ,'3_Poised_Promoter' ,'4_Strong_Enhancer', '5_Strong_Enhancer', '6_Weak_Enhancer', '7_Weak_Enhancer', '8_Insulator', '9_Txn_Transition', '10_Txn_Elongation', '11_Weak_Txn', '12_Repressed', '13_Heterochrom/lo', '14_Repetitive/CNV')

#' Function to recode classes from chromHMM type column
#'
#' @param hmm_codes in the original form from UCSC Genome Browser track.
#'
#' @return A character vector of chromHMM classes with numbers and underscores removed.
reformat_hmm_codes = function(hmm_codes) {
    new_codes = sapply(hmm_codes,
            function(hmm){paste(unlist(strsplit(hmm,'_'))[-1],collapse='')},
            USE.NAMES=FALSE)
    return(new_codes)
}

#' Function to return cell line from chromatin annotation shortcut
#'
#' @param shortcut The annotation shortcut, used in \code{build_annotations()}.
#'
#' @return A string of the cell line used in a chromatin annotation shortcut
get_cellline_from_shortcut = function(shortcut) {
    return(unlist(strsplit(unlist(strsplit(shortcut,'_'))[2], '-'))[1])
}

#' Function to return cell line from chromatin annotation code
#'
#' @param code The annotation code, used in \code{build_annotations()}.
#'
#' @return A string of the cell line used in a chromatin annotation code
get_cellline_from_code = function(code) {
    return(unlist(strsplit(unlist(strsplit(code,'_'))[3], '-'))[1])
}

#' Function listing which annotations are available.
#'
#' This includes the shortcuts. The \code{expand_annotations()} function helps
#' handle the shortcuts.
#'
#' @return A character vector of available annotations.
#'
#' @examples
#' builtin_annotations()
#'
#' @export
builtin_annotations = function() {
    # Create annotation code endings
        shortcut_ends = c('basicgenes','cpgs')

        # Gene codes
        gene_genomes = annotatr::builtin_genomes()
        gene_ends = c('1to5kb', 'promoters', 'cds', '5UTRs', 'exons', 'firstexons', 'introns', 'intronexonboundaries', 'exonintronboundaries', '3UTRs', 'intergenic')

        # CpG codes
        cpg_genomes = base::setdiff(annotatr::builtin_genomes(),c('dm3','dm6'))
        cpg_ends = c('islands', 'shores', 'shelves', 'inter')

        # Chromatin state codes
        # Remove numbers, and underscores, and take unique
        chromatin_recode = unique(reformat_hmm_codes(HMMCODES))

        chromatin_ends = apply(
            expand.grid(HMMCELLLINES, chromatin_recode, stringsAsFactors = FALSE),
            1, paste, collapse='-')

        chromatin_shortcut_ends = apply(
            expand.grid(HMMCELLLINES, 'chromatin', stringsAsFactors = FALSE),
            1, paste, collapse='-')

    # Create full annotation codes
        gene_codes = apply(
            expand.grid(gene_genomes, 'genes', gene_ends, stringsAsFactors = FALSE),
            1, paste, collapse='_')
        cpg_codes = apply(
            expand.grid(cpg_genomes, 'cpg', cpg_ends, stringsAsFactors= FALSE),
            1, paste, collapse='_')
        chromatin_codes = apply(
            expand.grid('hg19', 'chromatin', chromatin_ends, stringsAsFactors=FALSE),
            1, paste, collapse='_')

        enhancer_codes = c('hg19_enhancers_fantom','hg38_enhancers_fantom','mm9_enhancers_fantom','mm10_enhancers_fantom')
        lncrna_codes = c('hg19_lncrna_gencode','hg38_lncrna_gencode','mm10_lncrna_gencode')

        gene_shortcut_codes = apply(
            expand.grid(gene_genomes, 'basicgenes', stringsAsFactors = FALSE),
            1, paste, collapse='_')
        cpg_shortcut_codes = apply(
            expand.grid(cpg_genomes, 'cpgs', stringsAsFactors = FALSE),
            1, paste, collapse='_')
        chromatin_shortcut_codes = paste('hg19', chromatin_shortcut_ends, sep='_')

    # Create the big vector of supported annotations
    annots = c(gene_codes, cpg_codes, chromatin_codes, enhancer_codes, lncrna_codes,
        gene_shortcut_codes, cpg_shortcut_codes, chromatin_shortcut_codes)

    return(annots)
}

#' Function returning supported TxDb.* genomes
#'
#' @return A character vector of genomes for supported TxDb.* packages
#'
#' @examples
#' builtin_genomes()
#'
#' @export
builtin_genomes = function() {
    return(ORGDBS$genome)
}

#' Function to get correct TxDb.* package name based on genome
#'
#' @param genome A string giving the genome assembly.
#'
#' @return A string giving the name of the correct TxDb.* package name based on \code{genome}.
get_txdb_name = function(genome = annotatr::builtin_genomes()) {
    # Ensure valid arguments
    genome = match.arg(genome)

    db = grep(genome, TXDBS, value = TRUE)

    return(db)
}

#' Function to get correct org.* package name based on genome
#'
#' @param genome A string giving the genome assembly.
#'
#' @return A string giving the correct org for org.db packages. e.g. hg19 -> Hs.
get_orgdb_name = function(genome = annotatr::builtin_genomes()) {
    # Ensure valid arguments
    genome = match.arg(genome)

    org = ORGDBS[ORGDBS$genome == genome, 'org']

    return(org)
}

#' Function to tidy up annotation accessors for visualization
#'
#' @param annotations A character vector of annotations, in the order they are to appear in the visualization.
#'
#' @return A list of mappings from original annotation names to names ready for visualization.
#' @export
tidy_annotations = function(annotations) {
    tidy = sapply(annotations, function(a){
        tokens = unlist(strsplit(a,'_'))
        if(tokens[2] == 'cpg') {
            if(tokens[3] == 'inter') {
                return('interCGI')
            } else {
                return(paste('CpG', tokens[3]))
            }
        } else if (tokens[2] == 'genes') {
            if(tokens[3] == 'firstexons') {
                return('first exons')
            } else if (tokens[3] == 'intronexonboundaries') {
                return('intron/exon boundaries')
            } else if (tokens[3] == 'exonintronboundaries') {
                return('exon/intron boundaries')
            } else {
                return(tokens[3])
            }
        } else if (tokens[2] == 'enhancers') {
            return('enhancers')
        } else if (tokens[2] == 'chromatin') {
            return(tokens[3])
        } else if (tokens[2] == 'custom') {
            return(tokens[3])
        } else if (tokens[2] == 'lncrna') {
            return('GENCODE lncRNA')
        } else {
            return(sprintf('%s %s', tokens[2], tokens[3]))
        }
    })

    flip_tidy = names(tidy)
    names(flip_tidy) = tidy

    return(as.list(flip_tidy))
}

#' Function to check for valid annotations
#'
#' Gives errors if any annotations are not in builtin_annotations() (and they are not in the required custom format), basicgenes are used, or the genome prefixes are not the same for all annotations.
#'
#' @param annotations A character vector of annotations possibly using the shortcuts
#' @return If all the checks on the annotations pass, returns NULL to allow code to move forward.
check_annotations = function(annotations) {
    # Pull out any custom annotations before checking
    custom_annotations = grep('custom', annotations, value = TRUE)
    annotations = base::setdiff(annotations, custom_annotations)

    # Check that the annotations are supported, tell the user which are unsupported
    if( !all(annotations %in% annotatr::builtin_annotations()) ) {
        unsupported = base::setdiff(annotations, annotatr::builtin_annotations())

        stop(sprintf('Error: "%s" is(are) not supported. See builtin_annotations().',
            paste(unsupported, collapse=', ')))
    }

    # Recombine annotations and custom_annotations or you get failure when
    # there are only custom annotations
    annotations = c(custom_annotations, annotations)

    genomes = sapply(annotations, function(a){
        unlist(strsplit(a, '_'))[1]
    }, USE.NAMES = FALSE)

    # Check for same genome on all annotations
    if( length(unique(genomes)) != 1 ){
        stop('Error: genome prefix on all annotations must be the same.')
    }

    return(NULL)
}

#' Function to expand annotation shortcuts
#'
#' @param annotations A character vector of annotations, possibly using the shortcut accessors
#'
#' @return A vector of data accession-ized names that are ordered from upstream to downstream in the case of knownGenes and islands to interCGI in the case of cpgs.
#' @export
expand_annotations = function(annotations) {
    are_basicgenes = any(grepl('basicgenes', annotations))
    are_cpgs = any(grepl('cpgs', annotations))
    are_hmms = any(grepl('-chromatin', annotations))

    which_are_shortcuts = c(which(grepl('basicgenes', annotations)), which(grepl('cpgs', annotations)), which(grepl('-chromatin', annotations)))

    # expand_shortcuts() will always be run after check_annotations() so we can be
    # sure that the genome prefixes are the same for all annotaitons.
    genome = unique( sapply(annotations, function(a){ unlist(strsplit(a, '_'))[1] }, USE.NAMES = FALSE) )

    if(are_basicgenes || are_cpgs || are_hmms) {

        # Check for shortcut annotation accessors 'cpgs', 'basicgenes'
        # and create the right annotations based on the genome
        new_annotations = c()
        remove_shortcuts = c()
        if(are_cpgs) {
            new_annotations = paste(genome, 'cpg', c('islands','shores','shelves','inter'), sep='_')
        }
        if(are_basicgenes) {
            new_annotations = c(new_annotations, paste(genome, 'genes', c('1to5kb','promoters','5UTRs','exons','introns','3UTRs'), sep='_'))
        }
        if(are_hmms) {
            # Could conceivably use shortcuts for multiple cell lines
            hmms = grep('-chromatin', annotations, value = TRUE)
            cell_lines = sapply(hmms, get_cellline_from_shortcut, USE.NAMES = FALSE)

            new_hmm_codes = apply(
                expand.grid(cell_lines, unique(reformat_hmm_codes(HMMCODES)), stringsAsFactors = FALSE),
                1, paste, collapse='-')

            new_annotations = c(new_annotations,
                paste(genome, 'chromatin', new_hmm_codes, sep='_'))
        }
        annotations = base::setdiff(c(annotations, new_annotations), annotations[which_are_shortcuts])
    }

    return(annotations)
}

#' Function to subset a tbl_df or grouped_df by a column
#'
#' @param tbl A \code{tbl_df} or \code{grouped_df}.
#' @param col A string indicating which column of of \code{tbl} to subset and order
#' @param col_order A character vector indicating the order of \code{col}.
#'
#' @return A modified version of \code{summary} with \code{col} subsetted by \code{col_order}.
#' @export
subset_order_tbl = function(tbl, col, col_order) {
    if(!is.null(col)) {
        # Collect all types in the column
        all_col_names = unique(tbl[[col]])

        # Inherit col_order from the order in tbl
        if(is.null(col_order)) {
            col_order = all_col_names
        }

        # Check set equality of col in the summary and the col_order
        if( !dplyr::setequal(all_col_names, col_order) ) {
            if( all(col_order %in% all_col_names) ) {
                tbl = subset(tbl, tbl[[col]] %in% col_order)
            } else {
        # Intersect col_order with unique(tbl[[col]]) to deal with possible 0 tallies
        col_order = intersect(col_order, unique(tbl[[col]]))
                warning('There are elements in col_order that are not present in the corresponding column. Check for typos, or this could be a result of 0 tallies.')
            }
        }

        # Convert fill to factor with levels in the correct order
        tbl[[col]] = factor(tbl[[col]], levels = col_order)
        # Also convert the levels to tidy names if fill is annotations
        if(col == 'annot.type') {
            levels(tbl[[col]]) = tidy_annotations(col_order)
        }
    }
    return(tbl)
}

context('Test utility functions')

test_that('Test get_*_name() functions', {
    expect_error(
        get_txdb_name(genome = 'hg18'),
        'should be one of')

    expect_error(
        get_orgdb_name(genome = 'hg18'),
        'should be one of')
})

test_that('Test tidy_annotations()', {
    hg19_annots = c('hg19_cpg_islands', 'hg19_cpg_inter', 'hg19_genes_firstexons', 'hg19_genes_intronexonboundaries', 'hg19_genes_exonintronboundaries', 'hg19_lncrna_gencode', 'hg19_chromatin_Gm12878-ActivePromoter')
    mm9_annots = c('mm9_cpg_islands','mm9_genes_exonsCDSs','mm9_cpg_inter')
    rn4_custom_annots = c('rn4_custom_cpgislands','rn4_custom_TFBS')

    hg19_tidy_annots = tidy_annotations(hg19_annots)
    mm9_tidy_annots = tidy_annotations(mm9_annots)
    rn4_tidy_annots = tidy_annotations(rn4_custom_annots)

    expect_equal( all(names(hg19_tidy_annots) == c('CpG islands', 'interCGI', 'first exons', 'intron/exon boundaries', 'exon/intron boundaries', 'GENCODE lncRNA', 'Gm12878-ActivePromoter')), expected = TRUE)
    expect_equal( all(names(mm9_tidy_annots) == c('CpG islands', 'exonsCDSs', 'interCGI')), expected = TRUE)
    expect_equal( all(names(rn4_tidy_annots) == c('cpgislands', 'TFBS')), expected = TRUE)
    expect_equal
})

test_that('Test check_annotations()', {
    annots1 = c('hg17_genes_promoters','hg19_cpgs')
    annots2 = c('hello','hg19_genes_promoters','hg19_cpgs')
    annots3 = c('hg19_genes_promoters', 'mm9_cpg_islands')

    expect_error( check_annotations(annots1), 'not supported. See builtin_annotations()' )
    expect_error( check_annotations(annots2), 'not supported. See builtin_annotations()' )
    expect_error( check_annotations(annots3), 'genome prefix on all annotations must be the same' )
})

test_that('Test expand_annotations()', {
    annots1 = c('hg19_genes_promoters', 'hg19_genes_exons')

    annots2 = c('mm9_basicgenes', 'mm9_cpgs')
    expanded_annots2 = c('mm9_cpg_islands', 'mm9_cpg_shores', 'mm9_cpg_shelves', 'mm9_cpg_inter', 'mm9_genes_1to5kb', 'mm9_genes_promoters', 'mm9_genes_5UTRs', 'mm9_genes_exons', 'mm9_genes_introns', 'mm9_genes_3UTRs')

    annots3 = c('hg19_cpg_shores', 'hg19_cpgs')
    expanded_annots3 = c('hg19_cpg_islands', 'hg19_cpg_shores', 'hg19_cpg_shelves','hg19_cpg_inter')

    annots4 = c('hg19_Hepg2-chromatin')
    expanded_annots4 = c('hg19_chromatin_Hepg2-ActivePromoter','hg19_chromatin_Hepg2-WeakPromoter','hg19_chromatin_Hepg2-PoisedPromoter','hg19_chromatin_Hepg2-StrongEnhancer','hg19_chromatin_Hepg2-WeakEnhancer','hg19_chromatin_Hepg2-Insulator','hg19_chromatin_Hepg2-TxnTransition','hg19_chromatin_Hepg2-TxnElongation','hg19_chromatin_Hepg2-WeakTxn','hg19_chromatin_Hepg2-Repressed','hg19_chromatin_Hepg2-Heterochrom/lo','hg19_chromatin_Hepg2-Repetitive/CNV')

    expect_equal( dplyr::setequal(expand_annotations(annots1), annots1), expected = TRUE )
    expect_equal( dplyr::setequal(expand_annotations(annots2), expanded_annots2), expected = TRUE )
    expect_equal( dplyr::setequal(expand_annotations(annots3), expanded_annots3), expected = TRUE )
    expect_equal( dplyr::setequal(expand_annotations(annots4),
    expanded_annots4), expected = TRUE)
})

context('Test build annotations module')

################################################################################
# Test errors

test_that('Test error for non-existent custom annotations', {
    expect_error(
        build_annotations(genome = 'hg19', annotations = 'hg19_custom_ezh2'),
        'not in annotatr_cache'
    )
})

################################################################################
# Test annotations that aren't otherwise tested
# intergenic, cds, firstexons, and both boundaries

# test_that('Test all annotations', {
#     annots = c('hg19_basicgenes', 'hg19_cpgs', 'hg19_genes_intergenic', 'hg19_genes_cds', 'hg19_genes_firstexons', 'hg19_genes_intronexonboundaries', 'hg19_genes_exonintronboundaries', 'hg19_lncrna_gencode', 'hg19_Gm12878-chromatin', 'hg19_H1hesc-chromatin', 'hg19_Hepg2-chromatin', 'hg19_Hmec-chromatin', 'hg19_Hsmm-chromatin', 'hg19_Huvec-chromatin', 'hg19_K562-chromatin', 'hg19_Nhek-chromatin', 'hg19_Nhlf-chromatin')
#     annotations = build_annotations(genome = 'hg19', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('hg38_basicgenes', 'hg38_cpgs', 'hg38_genes_intergenic', 'hg38_genes_cds', 'hg38_genes_firstexons', 'hg38_genes_intronexonboundaries', 'hg38_genes_exonintronboundaries', 'hg38_lncrna_gencode', 'hg38_enhancers_fantom')
#     annotations = build_annotations(genome = 'hg38', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('mm10_basicgenes', 'mm10_cpgs', 'mm10_genes_intergenic', 'mm10_genes_cds', 'mm10_genes_firstexons', 'mm10_genes_intronexonboundaries', 'mm10_genes_exonintronboundaries', 'mm10_lncrna_gencode', 'mm10_enhancers_fantom')
#     annotations = build_annotations(genome = 'mm10', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('mm9_basicgenes', 'mm9_cpgs', 'mm9_genes_intergenic', 'mm9_genes_cds', 'mm9_genes_firstexons', 'mm9_genes_intronexonboundaries', 'mm9_genes_exonintronboundaries', 'mm9_enhancers_fantom')
#     annotations = build_annotations(genome = 'mm9', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('rn4_basicgenes', 'rn4_cpgs', 'rn4_genes_intergenic', 'rn4_genes_cds', 'rn4_genes_firstexons', 'rn4_genes_intronexonboundaries', 'rn4_genes_exonintronboundaries')
#     annotations = build_annotations(genome = 'rn4', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('rn5_basicgenes', 'rn5_cpgs', 'rn5_genes_intergenic', 'rn5_genes_cds', 'rn5_genes_firstexons', 'rn5_genes_intronexonboundaries', 'rn5_genes_exonintronboundaries')
#     annotations = build_annotations(genome = 'rn5', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('rn6_basicgenes', 'rn6_cpgs', 'rn6_genes_intergenic', 'rn6_genes_cds', 'rn6_genes_firstexons', 'rn6_genes_intronexonboundaries', 'rn6_genes_exonintronboundaries')
#     annotations = build_annotations(genome = 'rn6', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('dm3_basicgenes', 'dm3_genes_intergenic', 'dm3_genes_cds', 'dm3_genes_firstexons', 'dm3_genes_intronexonboundaries', 'dm3_genes_exonintronboundaries')
#     annotations = build_annotations(genome = 'dm3', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
#
#     annots = c('dm6_basicgenes', 'dm6_genes_intergenic', 'dm6_genes_cds', 'dm6_genes_firstexons', 'dm6_genes_intronexonboundaries', 'dm6_genes_exonintronboundaries')
#     annotations = build_annotations(genome = 'dm6', annotations = annots)
#     expect_true( dplyr::setequal(unique(annotations$type), expand_annotations(annots)) )
# })

#' A function to intersect user region data with annotation data
#'
#' Annotate genomic regions to selected genomic annotations while preserving the data associated with the genomic regions.
#'
#' @param regions The GRanges object returned by \code{read_regions()}.
#' @param annotations A character vector of annotations to build. Valid annotation codes are listed with \code{builtin_annotations()}. The "basicgenes" shortcut builds the following regions: 1-5Kb upstream of TSSs, promoters, 5UTRs, exons, introns, and 3UTRs. The "cpgs" shortcut builds the following regions: CpG islands, shores, shelves, and interCGI regions. NOTE: Shortcuts need to be appended by the genome, e.g. \code{hg19_basicgenes}.
#' Custom annotations whose names are of the form \code{[genome]_custom_[name]} should also be included. Custom annotations should be read in and converted to \code{GRanges} with \code{read_annotations()}. They can be for a \code{supported_genome()}, or for an unsupported genome.
#' @param minoverlap A scalar, positive integer, indicating the minimum required overlap of regions with annotations.
#' @param ignore.strand Logical indicating whether strandedness should be respected in findOverlaps(). Default FALSE.
#' @param quiet Print progress messages (FALSE) or not (TRUE).
#'
#' @return A \code{GRanges} where the \code{granges} are from the regions, and the \code{mcols} include the \code{mcols} from the regions and a column with the annotation \code{GRanges}.
#'
#' @examples
#'    r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
#'    extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
#'    r = read_regions(con = r_file, extraCols = extraCols, rename_score = 'coverage')
#'
#'    # Get premade CpG annotations
#'    data('annotations', package = 'annotatr')
#'
#'    a = annotate_regions(
#'        regions = r,
#'        annotations = annotations,
#'        ignore.strand = TRUE)
#'
#' @export
annotate_regions = function(regions, annotations, minoverlap = 1L, ignore.strand = TRUE, quiet = FALSE) {
    # Checks before moving forward
    if(class(regions)[1] != "GRanges") {
        stop('Error in annotate_regions(...): regions object is not GRanges.')
    }

    if(class(annotations)[1] != "GRanges") {
        stop('Error in annotate_regions(...): annotations object is not GRanges. Use build_annotations(...) to construct the annotations before calling annotate_regions(...).')
    }

    # Perform the intersections
    if(!quiet) {
        message('Annotating...')
    }

    intersections = GenomicRanges::findOverlaps(regions, annotations, minoverlap = minoverlap, ignore.strand = ignore.strand)

    if(length(intersections) > 0) {
        gr = regions[S4Vectors::queryHits(intersections)]
        GenomicRanges::mcols(gr)$annot = annotations[S4Vectors::subjectHits(intersections)]
        return(gr)
    } else {
        stop('No annotations intersect the regions.')
    }
}

#' annotatr: Annotation of Genomic Regions to Functional Annotations
#'
#' Given a set of genomic sites/regions (e.g. ChIP-seq peaks, CpGs, differentially methylated CpGs or regions, SNPs, etc.) it is often of interest to investigate the intersecting functional annotations. Such annotations include those relating to gene models (promoters, 5'UTRs, exons, introns, and 3'UTRs), CpGs (CpG islands, CpG shores, CpG shelves), the non-coding genome, and enhancers. The annotatr package provides an easy way to summarize and visualize the intersection of genomic sites/regions with the above functional annotations.
#'
#' @docType package
#' @name annotatr
#'
#' @rawNamespace import(AnnotationDbi, except='select')
#' @rawNamespace import(AnnotationHub, except='query')
#' @import dplyr
#' @import ggplot2
#' @importFrom rlang .data
#' @import GenomicFeatures
#' @rawNamespace import(GenomicRanges, except=c('union','setdiff','intersect','union'))
#' @importClassesFrom Seqinfo Seqinfo
#' @importFrom Seqinfo seqnames seqlengths
#' @importFrom IRanges IRanges
#' @importFrom S4Vectors endoapply
#' @importFrom S4Vectors splitAsList
#' @import methods
#' @importFrom readr read_tsv
#' @importFrom reshape2 melt
#' @importFrom regioneR randomizeRegions
#' @importFrom rtracklayer import import.bed
#' @importClassesFrom S4Vectors Hits Rle
#' @importFrom stats as.formula
#' @importFrom utils combn data
NULL

#' A global-variable to hold custom annotations loaded in an R session
#'
#' Code thanks to Martin Morgan. This is a global variable that will store custom
#' annotations that a user reads in during a session in which annotatr is loaded.
#'
#' @return An environment to contain custom annotations from \code{read_annotations}.
#'
#' @examples
#'  # Example usage
#'  annotatr_cache$set("foo", 1:10)
#'  annotatr_cache$get("foo")
#'
#'  # Read in a BED3 file as a custom annotation
#'  file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
#'  # The custom annotation is added to the annotatr_cache environment in this function
#'  read_annotations(con = file, name = 'test', genome = 'hg19')
#'  # The result of read_annotations() is not visible in .GlobalEnv, instead
#'  # need to use the get method
#'  print(annotatr_cache$get('hg19_custom_test'))
#'  # See what is in the annotatr_cache
#'  annotatr_cache$list_env()
#' @export
annotatr_cache <- local({
    env = new.env(parent = emptyenv())
    list(set=function(key, value) {
        env[[key]] = value
        invisible(value)
    }, get = function(key) {
        if (!exists(key, env))
            stop(sQuote(key), " not in annotatr_cache", call.=FALSE)
        env[[key]]
    }, list_env=function() {
        ls(env)
    })
})

#' A function to build annotations from TxDb.* and AnnotationHub resources
#'
#' Create a \code{GRanges} object consisting of all the desired \code{annotations}. Supported annotation codes are listed by \code{builtin_annotations()}. The basis for enhancer annotations are FANTOM5 data, the basis for CpG related annotations are CpG island tracks from \code{AnnotationHub}, and the basis for genic annotations are from the \code{TxDb.*} and \code{org.db} group of packages.
#'
#' @param genome The genome assembly.
#' @param annotations A character vector of annotations to build. Valid annotation codes are listed with \code{builtin_annotations()}. The "basicgenes" shortcut builds the following regions: 1-5Kb upstream of TSSs, promoters, 5UTRs, exons, introns, and 3UTRs. The "cpgs" shortcut builds the following regions: CpG islands, shores, shelves, and interCGI regions. NOTE: Shortcuts need to be appended by the genome, e.g. \code{hg19_basicgenes}.
#' Custom annotations whose names are of the form \code{[genome]_custom_[name]} should also be included. Custom annotations should be read in and converted to \code{GRanges} with \code{read_annotations()}. They can be for a \code{supported_genome()}, or for an unsupported genome.
#'
#' @return A \code{GRanges} object of all the \code{annotations} combined. The \code{mcols} are \code{id, tx_id, gene_id, symbol, type}. The \code{id} column is a unique name, the \code{tx_id} column is either a UCSC knownGene transcript ID (genic annotations) or a Ensembl transcript ID (lncRNA annotations), the \code{gene_id} is the Entrez ID, the \code{symbol} is the gene symbol from the \code{org.*.eg.db} mapping from the Entrez ID, and the \code{type} is of the form \code{[genome]_[type]_[name]}.
#'
#' @examples
#' # Example with hg19 gene promoters
#' annots = c('hg19_genes_promoters')
#' annots_gr = build_annotations(genome = 'hg19', annotations = annots)
#'
#' # See vignette for an example with custom annotation
#'
#' @export
build_annotations = function(genome, annotations) {
    # Expand annotations first
    annotations = expand_annotations(annotations)

    # Collect built-in annotations
    hmm_annotations = grep('_chromatin_', annotations, value=TRUE)
    enh_annotations = grep('_enhancers_', annotations, value=TRUE)
    gene_annotations = grep('_genes_', annotations, value=TRUE)
    cpg_annotations = grep('_cpg_', annotations, value=TRUE)
    lncrna_annotations = grep('_lncrna_', annotations, value=TRUE)
    builtin_annotations = c(hmm_annotations, enh_annotations, gene_annotations, cpg_annotations, lncrna_annotations)

    # Check builtin_annotations
    if(length(builtin_annotations) > 0) {
        check_annotations(builtin_annotations)
    }

    # Collect custom and AnnotationHub annotations
    other_annotations = setdiff(annotations, builtin_annotations)

    # Container for the annotations
    annots_grl = GenomicRanges::GRangesList()

    # Do the other_annotations first because we don't want to fail unexpectedly at the end
    if(length(other_annotations) > 0) {
        annots_grl = c(annots_grl, GenomicRanges::GRangesList(sapply(other_annotations, function(ca){annotatr_cache$get(ca)})))
    }
    # Take the builtin_annotations piece by piece
    if(length(enh_annotations) != 0) {
        annots_grl = c(annots_grl, GenomicRanges::GRangesList(enhancers_fantom = suppressWarnings(build_enhancer_annots(genome = genome))))
    }
    if(length(hmm_annotations) != 0) {
        annots_grl = c(annots_grl, GenomicRanges::GRangesList(chromatin = suppressWarnings(build_hmm_annots(genome = genome, annotations = hmm_annotations))))
    }
    if(length(gene_annotations) != 0) {
        annots_grl = c(annots_grl, suppressWarnings(build_gene_annots(genome = genome, annotations = gene_annotations)))
    }
    if(length(cpg_annotations) != 0) {
        annots_grl = c(annots_grl, suppressWarnings(build_cpg_annots(genome = genome, annotations = cpg_annotations)))
    }
    if(length(lncrna_annotations) != 0) {
        annots_grl = c(annots_grl, GenomicRanges::GRangesList(lncrna_gencode = suppressWarnings(build_lncrna_annots(genome = genome))))
    }

    gr = unlist(annots_grl, use.names=FALSE)
    names(gr) = NULL

    return(gr)
}

#' A helper function to build arbitrary annotatinos from AnnotationHub
#'
#' @param genome The genome assembly.
#' @param ah_codes A named character vector giving the AnnotationHub accession number (e.g. AH23256), and whose name describes what the annotation is (e.g. Gm12878_H3K4me3).
#' @param annotation_class A string to name the group of annotations in \code{ah_codes}
#'
#' @return A \code{GRanges} object stored in \code{annotatr_cache}. To view an annotation built with this function, do \code{annotatr_cache$get(name)}. To add these annotations to a set of annotations, include \code{'[genome]_[annotation_class]_[name]'} in the call to \code{build_annotations()}. See example below.
#' @examples
#'
#' # Create a named vector for the AnnotationHub accession codes with desired names
#' h3k4me3_code = c('Gm12878' = 'AH23256')
#' # Fetch ah_codes from AnnotationHub and create annotations annotatr understands
#' build_ah_annots(genome = 'hg19', ah_codes = h3k4me3_code, annotation_class = 'H3K4me3')
#' # The annotations as they appear in annotatr_cache
#' annot_name = c('hg19_H3K4me3_Gm12878')
#' # Build the annotations right before annotating any regions
#' annotations = build_annotations(genome = 'hg19', annotations = annot_name)
#'
#' @export
build_ah_annots = function(genome, ah_codes, annotation_class) {
    # Check that all ah_codes have non-empty names
    ah_names = names(ah_codes)
    if(any(ah_names == '')) {
        stop('All ah_codes should have non-empty names.')
    }

    # Create an AnnotationHub instance and get all the accession codes
    ah = AnnotationHub::AnnotationHub()
    all_ah_codes = names(ah)

    # Check that ah_codes are valid and throw informative error
    invalid_ah_codes = setdiff(ah_codes, all_ah_codes)
    if(length(invalid_ah_codes) > 0) {
        stop(sprintf('The following ah_codes are invalid: %s.', paste(invalid_ah_codes, collapse=', ')))
    }

    # Check that ah_codes are GRanges and throw informative error
    ah_rdataclass = ah[ah_codes]$rdataclass
    if(any(ah_rdataclass != 'GRanges')) {
        non_rdataclass = which(ah_rdataclass != 'GRanges')
        stop(sprintf('The following ah_codes are not GRanges: %s. Only AnnotationHub accessions with rdataclass == GRanges are allowed.', paste(ah_codes[non_rdataclass], collapse=', ')))
    }

    for(i in seq_along(ah_codes)) {
        message(sprintf('Building annotation %s from AnnotationHub resource %s ...', names(ah_codes)[i], ah_codes[i]))
        gr = ah[[ah_codes[i]]]
        gr = GenomicRanges::granges(gr)
        gr = GenomicRanges::sort(gr)

        GenomicRanges::mcols(gr)$id = paste0(sprintf('%s_%s:', annotation_class, names(ah_codes[i])), seq_along(gr))
        GenomicRanges::mcols(gr)$tx_id = NA
        GenomicRanges::mcols(gr)$gene_id = NA
        GenomicRanges::mcols(gr)$symbol = NA
        GenomicRanges::mcols(gr)$type = sprintf('%s_%s_%s', genome, annotation_class, names(ah_codes[i]))

        GenomicRanges::mcols(gr) = GenomicRanges::mcols(gr)[, c('id','tx_id','gene_id','symbol','type')]

        ########################################################
        # Write the object named [genome]_[annotation_class]_[name] to the annotatr_cache
        annotatr_cache$set(sprintf('%s_%s_%s', genome, annotation_class, names(ah_codes[i])), gr)
    }
}

#' A helper function to build chromHMM annotations for hg19 from UCSC Genome Browser.
#'
#' @param genome The genome assembly.
#' @param annotations A character vector of valid chromatin state annotatin codes.
#'
#' @return A \code{GRanges} object.
build_hmm_annots = function(genome = c('hg19'), annotations = annotatr::builtin_annotations()) {
    # Ensure valid arguments
    genome = match.arg(genome)
    annotations = match.arg(annotations, several.ok = TRUE)

    message('Building hmms...')

    cell_lines = unique(sapply(annotations, get_cellline_from_code, USE.NAMES = FALSE))

    # If chromHMMs for different cell lines are requested, need to get them all
    hmms_list = GenomicRanges::GRangesList(lapply(cell_lines, function(line){
        message(sprintf('Downloading chromHMM track for %s', line))
        # Fetch the correct information for the line
        con = sprintf('http://hgdownload.cse.ucsc.edu/goldenpath/%s/database/wgEncodeBroadHmm%sHMM.txt.gz', genome, line)
        # Read from URL. There is surprisingly nothing in base that
        # does this as easily, so here we are with readr again.
        tbl = readr::read_tsv(con,
            col_names = c('chr','start','end','type'),
            col_types = '-ciic-----')

        # Reformat types
        types = sprintf('%s_chromatin_%s', genome, paste(line, reformat_hmm_codes(tbl$type), sep='-'))

        # Convert to GRanges
        gr = tryCatch({
            GenomicRanges::GRanges(
            seqnames = tbl$chr,
            ranges = IRanges::IRanges(start = tbl$start, end = tbl$end),
            strand = '*',
            type = types,
            seqinfo = Seqinfo::Seqinfo(genome=genome))
        }, error = function(e) {
            GenomicRanges::GRanges(
            seqnames = tbl$chr,
            ranges = IRanges::IRanges(start = tbl$start, end = tbl$end),
            strand = '*',
            type = types)
        })

        return(gr)
    }))
    hmms_gr = unlist(hmms_list, use.names = FALSE)

    # Split the GRanges into a GRangesList for subsetting and id population
    hmms_grl = S4Vectors::splitAsList(hmms_gr, GenomicRanges::mcols(hmms_gr)$type)

    hmms_grl = hmms_grl[names(hmms_grl) %in% annotations]

    hmms_grl = GenomicRanges::GRangesList(lapply(names(hmms_grl), function(n) {
        gr = hmms_grl[[n]]
        gr$id = paste0(n,':',seq_along(gr))
        return(gr)
    }))

    hmms_gr = unlist(hmms_grl, use.names=FALSE)

    GenomicRanges::mcols(hmms_gr)$tx_id = NA
    GenomicRanges::mcols(hmms_gr)$gene_id = NA
    GenomicRanges::mcols(hmms_gr)$symbol = NA

    GenomicRanges::mcols(hmms_gr) = GenomicRanges::mcols(hmms_gr)[, c('id','tx_id','gene_id','symbol','type')]

    return(hmms_gr)
}

#' A helper function to build enhancer annotations for hg19 and mm10 from FANTOM5.
#'
#' @param genome The genome assembly.
#'
#' @return A \code{GRanges} object.
build_enhancer_annots = function(genome = c('hg19','hg38','mm9','mm10')) {
    # Ensure valid arguments
    genome = match.arg(genome)

    message('Building enhancers...')

    # NOTE: Since something like hg38_enhancers_fantom will be caught by check_annotations() there is no need to worry about anything other than hg19 or mm10 will get to this point.

    # Get the enhancer annotations from FANTOM5
    if(genome == 'hg19') {
        enhancers = rtracklayer::import.bed('http://fantom.gsc.riken.jp/5/datafiles/phase2.0/extra/Enhancers/human_permissive_enhancers_phase_1_and_2.bed.gz', genome = 'hg19')
    } else if(genome == 'hg38') {
        # Create AnnotationHub connection
        ah = AnnotationHub::AnnotationHub()
        chain = ah[['AH14150']]

        # Get hg19 enhancers
        hg19_enhancers = rtracklayer::import.bed('http://fantom.gsc.riken.jp/5/datafiles/phase2.0/extra/Enhancers/human_permissive_enhancers_phase_1_and_2.bed.gz', genome = 'hg19')

        enhancers = rtracklayer::liftOver(x = hg19_enhancers, chain = chain)
        enhancers = sort(unlist(enhancers))
    } else if (genome == 'mm9') {
        enhancers = rtracklayer::import.bed('http://fantom.gsc.riken.jp/5/datafiles/phase2.0/extra/Enhancers/mouse_permissive_enhancers_phase_1_and_2.bed.gz', genome = 'mm9')
    } else if (genome == 'mm10') {
        # Create AnnotationHub connection
        ah = AnnotationHub::AnnotationHub()
        chain = ah[['AH14596']]

        # Get hg19 enhancers
        mm9_enhancers = rtracklayer::import.bed('http://fantom.gsc.riken.jp/5/datafiles/phase2.0/extra/Enhancers/mouse_permissive_enhancers_phase_1_and_2.bed.gz', genome = 'mm9')

        enhancers = rtracklayer::liftOver(x = mm9_enhancers, chain = chain)
        enhancers = sort(unlist(enhancers))
    }

    enhancers = GenomicRanges::granges(enhancers)
    GenomicRanges::mcols(enhancers)$id = paste0('enhancer:', seq_along(enhancers))
    GenomicRanges::mcols(enhancers)$tx_id = NA
    GenomicRanges::mcols(enhancers)$gene_id = NA
    GenomicRanges::mcols(enhancers)$symbol = NA
    GenomicRanges::mcols(enhancers)$type = sprintf('%s_enhancers_fantom', genome)

    return(enhancers)
}

#' A helper function to build CpG related annotations.
#'
#' Using the \code{AnnotationHub} package, extract CpG island track for the appropriate \code{genome} and construct the shores, shelves, and interCGI annotations as desired.
#'
#' @param genome The genome assembly.
#' @param annotations A character vector with entries of the form \code{[genome]_cpg_{islands,shores,shelves,inter}}.
#'
#' @return A list of \code{GRanges} objects.
build_cpg_annots = function(genome = annotatr::builtin_genomes(), annotations = annotatr::builtin_annotations()) {
    # Ensure valid arguments
    genome = match.arg(genome)
    annotations = match.arg(annotations, several.ok = TRUE)

    annot_codes = data.frame(
        code = c(sprintf('%s_cpg_islands', genome),
            sprintf('%s_cpg_shores', genome),
            sprintf('%s_cpg_shelves', genome),
            sprintf('%s_cpg_inter', genome)),
        var = c('islands','shores','shelves','inter_cgi'),
        stringsAsFactors = FALSE)

    # Decide whether to use URL or AnnotationHub
    ah_genomes = c('hg19','mm9','rn5','rn4')
    if(genome == 'hg19' || genome == 'mm9' || genome == 'rn5' || genome == 'rn4') {
        use_ah = TRUE
    } else if (genome == 'hg38') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/hg38/database/cpgIslandExt.txt.gz'
    } else if (genome == 'mm10') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/mm10/database/cpgIslandExt.txt.gz'
    } else if (genome == 'mm39') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/mm39/database/cpgIslandExt.txt.gz'
    } else if (genome == 'rn6') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/rn6/database/cpgIslandExt.txt.gz'
    } else if (genome == 'rn7') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/rn7/database/cpgIslandExt.txt.gz'
    } else if (genome == 'danRer10') {
        use_ah = FALSE
        con = 'http://hgdownload.soe.ucsc.edu/goldenPath/danRer10/database/cpgIslandExt.txt.gz'
    } else if (genome == 'danRer11') {
        use_ah = FALSE
        con = 'http://hgdownload.soe.ucsc.edu/goldenPath/danRer11/database/cpgIslandExt.txt.gz'
    } else if (genome == 'galGal5') {
        use_ah = FALSE
        con = 'http://hgdownload.cse.ucsc.edu/goldenpath/galGal5/database/cpgIslandExt.txt.gz'
    } else {
        stop(sprintf('CpG features are not supported for genome %s', genome))
    }

    if(use_ah) {
        # Create AnnotationHub connection
        ah = AnnotationHub::AnnotationHub()

        # And do the query for available CpG Islands
        query = AnnotationHub::query(ah, c('CpG Islands'))

        # Determine the correct ID to extract data from AnnotationHub
        ID = row.names(GenomicRanges::mcols(query)[GenomicRanges::mcols(query)$genome == genome, ])
    }

    if(any(grepl('islands', annotations)) || any(grepl('shores', annotations)) || any(grepl('shelves', annotations)) || any(grepl('inter', annotations))) {
        message('Building CpG islands...')
        ### Islands
            # Extract and sort the islands based on use_ah
            if(use_ah) {
                islands = ah[[ID]]
            } else {
                # Read from URL. There is surprisingly nothing in base that
                # does this as easily, so here we are with readr again.
                islands_tbl = readr::read_tsv(con,
                    col_names = c('chr','start','end'),
                    col_types = '-cii-------')
                # Convert to GRanges
                islands = tryCatch({
                    GenomicRanges::GRanges(
                        seqnames = islands_tbl$chr,
                        ranges = IRanges::IRanges(start = islands_tbl$start, end = islands_tbl$end),
                        strand = '*',
                        seqinfo = Seqinfo::Seqinfo(genome=genome))
                }, error = function(e){
                    GenomicRanges::GRanges(
                        seqnames = islands_tbl$chr,
                        ranges = IRanges::IRanges(start = islands_tbl$start, end = islands_tbl$end),
                        strand = '*')
                })
            }
            islands = GenomicRanges::sort(islands)


            # Rename the islands
            GenomicRanges::mcols(islands)$id = paste0('island:', seq_along(islands))

            # Add tx_name, gene_id, and symbol columns
            GenomicRanges::mcols(islands)$tx_id = NA
            GenomicRanges::mcols(islands)$gene_id = NA
            GenomicRanges::mcols(islands)$symbol = NA

            # Give it the correct type
            GenomicRanges::mcols(islands)$type = sprintf('%s_cpg_islands', genome)

            GenomicRanges::mcols(islands) = GenomicRanges::mcols(islands)[, c('id','tx_id','gene_id','symbol','type')]

        if(any(grepl('shores', annotations)) || any(grepl('shelves', annotations)) || any(grepl('inter', annotations))) {
            message('Building CpG shores...')
            ### Shores
                # Construct the shores based on:
                # upstream from the island start and downstream from the island end
                up_shores = GenomicRanges::flank(islands, width = 2000, start = TRUE, both = FALSE)
                down_shores = GenomicRanges::flank(islands, width = 2000, start = FALSE, both = FALSE)

                # Combine, sort, trim, and reduce combined up_shores and down_shores
                shores = c(up_shores, down_shores)
                shores = GenomicRanges::sort(shores)
                shores = GenomicRanges::trim(shores)
                shores = GenomicRanges::reduce(shores)

                # Remove islands from the shores
                shores = GenomicRanges::setdiff(shores, islands)

                # Rename the shores
                GenomicRanges::mcols(shores)$id = paste0('shore:', seq_along(shores))

                # Add tx_name, gene_id, and symbol columns
                GenomicRanges::mcols(shores)$tx_id = NA
                GenomicRanges::mcols(shores)$gene_id = NA
                GenomicRanges::mcols(shores)$symbol = NA

                # Give it the correct type
                GenomicRanges::mcols(shores)$type = sprintf('%s_cpg_shores', genome)

                GenomicRanges::mcols(shores) = GenomicRanges::mcols(shores)[, c('id','tx_id','gene_id','symbol','type')]

            if(any(grepl('shelves', annotations)) || any(grepl('inter', annotations))) {
                message('Building CpG shelves...')
                ### Shelves
                    # Construct the shelves based on:
                    # upstream from the up_shores start and downstream from the down_shores end
                    up_shelves = GenomicRanges::flank(shores, width = 2000, start = TRUE, both = FALSE)
                    down_shelves = GenomicRanges::flank(shores, width = 2000, start = FALSE, both = FALSE)

                    # Combine, sort, trim, and reduce combined up_shelves and down_shelves
                    shelves = c(up_shelves, down_shelves)
                    shelves = GenomicRanges::sort(shelves)
                    shelves = GenomicRanges::trim(shelves)
                    shelves = GenomicRanges::reduce(shelves)

                    # Remove islands and/or shores from the shelves
                    shelves = GenomicRanges::setdiff(shelves, islands)
                    shelves = GenomicRanges::setdiff(shelves, shores)

                    # Rename the shelves
                    GenomicRanges::mcols(shelves)$id = paste0('shelf:', seq_along(shelves))

                    # Add gene_id, and symbol columns
                    GenomicRanges::mcols(shelves)$tx_id = NA
                    GenomicRanges::mcols(shelves)$gene_id = NA
                    GenomicRanges::mcols(shelves)$symbol = NA

                    # Give it the correct type
                    GenomicRanges::mcols(shelves)$type = sprintf('%s_cpg_shelves', genome)

                    GenomicRanges::mcols(shelves) = GenomicRanges::mcols(shelves)[, c('id','tx_id','gene_id','symbol','type')]

                if(any(grepl('inter', annotations))) {
                    message('Building inter-CpG-islands...')
                    ### interCGI
                        # Take the union of all the objects so far
                        extended_cgi = Reduce(
                            function(x,y){
                                GenomicRanges::union(x,y)
                            },
                            list(islands, shores, shelves)
                        )

                        # A quirk of GenomicRanges::gaps() on unstranded ranges gives the whole
                        # chroms on the + and - strands in addition to the * gaps. Remove +/- gaps.
                        inter_cgi = GenomicRanges::gaps(extended_cgi)
                        inter_cgi = inter_cgi[GenomicRanges::strand(inter_cgi) == '*']

                        # Rename the interCGI
                        GenomicRanges::mcols(inter_cgi)$id = paste0('inter:', seq_along(inter_cgi))

                        # Add gene_id, and symbol columns
                        GenomicRanges::mcols(inter_cgi)$tx_id = NA
                        GenomicRanges::mcols(inter_cgi)$gene_id = NA
                        GenomicRanges::mcols(inter_cgi)$symbol = NA

                        # Give it the correct type
                        GenomicRanges::mcols(inter_cgi)$type = sprintf('%s_cpg_inter', genome)

                        GenomicRanges::mcols(inter_cgi) = GenomicRanges::mcols(inter_cgi)[, c('id','tx_id','gene_id','symbol','type')]
                }
            }
        }
    }

    ### Put it all together
    mgets = annot_codes[annot_codes$code %in% annotations, 'var']
    cpgs = do.call('GRangesList', mget(mgets))
    names(cpgs) = annotations

    return(cpgs)
}

#' A helper function to build genic annotations.
#'
#' Using the \code{TxDb.*} group of packages, construct genic annotations consisting of any combination of 1-5kb upstream of a TSS, promoters (< 1kb from TSS), 5UTRs, CDS, exons, first exons, introns, intron/exon and exon/intron boundaries, 3UTRs, and intergenic.
#'
#' @param genome The genome assembly.
#' @param annotations A character vector with entries of the form \code{[genome]_genes_{1to5kb,promoters,5UTRs,cds,exons,firstexons,introns,intronexonboundaries,exonintronboundaries,3UTRs,intergenic}}.
#'
#' @return A list of \code{GRanges} objects with unique \code{id} of the form \code{[type]:i}, \code{tx_id} being the UCSC knownGene transcript name, \code{gene_id} being the Entrez Gene ID, \code{symbol} being the gene symbol from the Entrez ID to symbol mapping in \code{org.db} for that species, and \code{type} being the annotation type.
build_gene_annots = function(genome = annotatr::builtin_genomes(), annotations = annotatr::builtin_annotations()) {
    # Ensure valid arguments
    genome = match.arg(genome)
    annotations = match.arg(annotations, several.ok = TRUE)

    annot_codes = data.frame(
        code = c(sprintf('%s_genes_promoters', genome),
            sprintf('%s_genes_1to5kb', genome),
            sprintf('%s_genes_cds', genome),
            sprintf('%s_genes_5UTRs', genome),
            sprintf('%s_genes_exons', genome),
            sprintf('%s_genes_firstexons', genome),
            sprintf('%s_genes_introns', genome),
            sprintf('%s_genes_intronexonboundaries', genome),
            sprintf('%s_genes_exonintronboundaries', genome),
            sprintf('%s_genes_3UTRs', genome),
            sprintf('%s_genes_intergenic', genome)),
        var = c('promoters_gr','onetofive_gr','cds_gr','fiveUTRs_gr','exons_gr',
            'firstexons_gr','introns_gr','intronexon_gr','exonintron_gr',
            'threeUTRs_gr','intergenic_gr'),
        stringsAsFactors = FALSE)

    # Load the appropriate TxDb.* library and get the txdb
    err_mess = NULL
    txdb_name = get_txdb_name(genome)
    if(requireNamespace(txdb_name, quietly = TRUE)) {
        library(txdb_name, character.only = TRUE)
    } else {
        err_mess = sprintf('The package %s is not installed, please install it via Bioconductor.', txdb_name)
    }
    txdb = get(txdb_name)

    # Get the org.XX.eg.db mapping from Entrez ID to gene symbol
    # First element returned is package name, second is eg2SYMBOL name
    orgdb_name = get_orgdb_name(genome)
    if(requireNamespace(sprintf('org.%s.eg.db', orgdb_name), quietly = TRUE)) {
        library(sprintf('org.%s.eg.db', orgdb_name), character.only = TRUE)
    } else {
        err_mess = paste(
            err_mess,
            sprintf('The package org.%s.eg.db is not installed, please install it via Bioconductor.', orgdb_name),
            sep='\n')
    }
    if(!is.null(err_mess)) {
        stop(err_mess)
    }
    x = get(sprintf('org.%s.egSYMBOL', orgdb_name))
    mapped_genes = mappedkeys(x)
    eg2symbol = as.data.frame(x[mapped_genes])

    # Build the base transcripts
    tx_gr = transcripts(txdb, columns = c('TXID','GENEID','TXNAME'))
    # Create TSS GRanges for later use with intronexon boundaries
    tss_gr = GenomicRanges::GRanges(
        seqnames = seqnames(tx_gr),
        ranges = IRanges(start = start(tx_gr), end = start(tx_gr)),
        strand = '*'
    )
    GenomicRanges::mcols(tss_gr) = GenomicRanges::mcols(tx_gr)
    seqinfo(tss_gr) = seqinfo(tx_gr)

    # Create TES GRanges for later use with exonintron boundaries
    tes_gr = GenomicRanges::GRanges(
        seqnames = seqnames(tx_gr),
        ranges = IRanges(start = end(tx_gr), end = end(tx_gr)),
        strand = '*'
    )
    GenomicRanges::mcols(tes_gr) = GenomicRanges::mcols(tx_gr)
    seqinfo(tes_gr) = seqinfo(tx_gr)

    # Build tables to map TXID to TXNAME and GENEID
    id_maps = AnnotationDbi::select(txdb, keys = as.character(GenomicRanges::mcols(tx_gr)$TXID), columns = c('TXNAME','GENEID'), keytype = 'TXID')

    # Each annotation should be a GRanges object with the following mcols:
    # id, tx_id, gene_id, symbol, type

    if(any(grepl('promoters', annotations)) || any(grepl('1to5kb', annotations)) || any(grepl('intergenic', annotations))) {
        message('Building promoters...')
        ### promoters
            promoters_gr = GenomicFeatures::promoters(txdb, upstream = 1000, downstream = 0)
            # Add Entrez ID, symbol, and type
            GenomicRanges::mcols(promoters_gr)$gene_id = id_maps[match(GenomicRanges::mcols(promoters_gr)$tx_id, id_maps$TXID), 'GENEID']
            GenomicRanges::mcols(promoters_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(promoters_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(promoters_gr)$type = sprintf('%s_genes_promoters', genome)
            GenomicRanges::mcols(promoters_gr)$id = paste0('promoter:', seq_along(promoters_gr))

            GenomicRanges::mcols(promoters_gr) = GenomicRanges::mcols(promoters_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(promoters_gr)) = c('id','tx_id','gene_id','symbol','type')

        if(any(grepl('1to5kb', annotations)) || any(grepl('intergenic', annotations))) {
            message('Building 1to5kb upstream of TSS...')
            ### 1-5kb
                onetofive_gr = GenomicRanges::flank(promoters_gr, width = 4000, start = TRUE, both = FALSE)
                onetofive_gr = GenomicRanges::trim(onetofive_gr)
                # Add Entrez ID, symbol, and type (all but type are inherited from promoters_gr)
                GenomicRanges::mcols(onetofive_gr)$id = paste0('1to5kb:', seq_along(onetofive_gr))
                GenomicRanges::mcols(onetofive_gr)$type = sprintf('%s_genes_1to5kb', genome)

            if(any(grepl('intergenic', annotations))) {
                message('Building intergenic...')
                ### intergenic
                    genic_gr = c(GenomicRanges::granges(tx_gr), GenomicRanges::granges(promoters_gr), GenomicRanges::granges(onetofive_gr))
                    GenomicRanges::strand(genic_gr) = '*'
                    intergenic_gr = GenomicRanges::reduce(genic_gr)
                    intergenic_gr = GenomicRanges::gaps(intergenic_gr)

                    # A quirk in gaps gives the entire + and - strand of a chromosome, ignore those
                    intergenic_gr = intergenic_gr[GenomicRanges::strand(intergenic_gr) == '*']

                    GenomicRanges::mcols(intergenic_gr)$id = paste0('intergenic:', seq_along(intergenic_gr))
                    GenomicRanges::mcols(intergenic_gr)$tx_id = NA
                    GenomicRanges::mcols(intergenic_gr)$gene_id = NA
                    GenomicRanges::mcols(intergenic_gr)$symbol = NA
                    GenomicRanges::mcols(intergenic_gr)$type = sprintf('%s_genes_intergenic', genome)
            }
        }
    }

    if(any(grepl('cds', annotations))) {
        message('Building cds...')
        ### cds
            cds_grl = GenomicFeatures::cdsBy(txdb, by = 'tx', use.names = TRUE)
            # Create Rle of the tx_names
            cds_txname_rle = S4Vectors::Rle(names(cds_grl), S4Vectors::elementNROWS(cds_grl))
            cds_txname_vec = as.character(cds_txname_rle)
            # Unlist and add the tx_names
            cds_gr = unlist(cds_grl, use.names = FALSE)
            GenomicRanges::mcols(cds_gr)$tx_name = cds_txname_vec
            # Add Entrez ID, symbol, and type
            GenomicRanges::mcols(cds_gr)$gene_id = id_maps[match(GenomicRanges::mcols(cds_gr)$tx_name, id_maps$TXNAME), 'GENEID']
            GenomicRanges::mcols(cds_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(cds_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(cds_gr)$type = sprintf('%s_genes_cds', genome)
            GenomicRanges::mcols(cds_gr)$id = paste0('CDS:', seq_along(cds_gr))

            GenomicRanges::mcols(cds_gr) = GenomicRanges::mcols(cds_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(cds_gr)) = c('id','tx_id','gene_id','symbol','type')
    }

    if(any(grepl('5UTR', annotations))) {
        message('Building 5UTRs...')
        ### fiveUTRs
            fiveUTRs_grl = GenomicFeatures::fiveUTRsByTranscript(txdb, use.names = TRUE)
            # Create Rle of the tx_names
            fiveUTRs_txname_rle = S4Vectors::Rle(names(fiveUTRs_grl), S4Vectors::elementNROWS(fiveUTRs_grl))
            fiveUTRs_txname_vec = as.character(fiveUTRs_txname_rle)
            # Unlist and add the tx_names
            fiveUTRs_gr = unlist(fiveUTRs_grl, use.names = FALSE)
            GenomicRanges::mcols(fiveUTRs_gr)$tx_name = fiveUTRs_txname_vec
            # Add Entrez ID, symbol, and type
            # NOTE: here we match on the tx_name because the tx_id is not given
            GenomicRanges::mcols(fiveUTRs_gr)$gene_id = id_maps[match(GenomicRanges::mcols(fiveUTRs_gr)$tx_name, id_maps$TXNAME), 'GENEID']
            GenomicRanges::mcols(fiveUTRs_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(fiveUTRs_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(fiveUTRs_gr)$type = sprintf('%s_genes_5UTRs', genome)
            GenomicRanges::mcols(fiveUTRs_gr)$id = paste0('5UTR:', seq_along(fiveUTRs_gr))

            GenomicRanges::mcols(fiveUTRs_gr) = GenomicRanges::mcols(fiveUTRs_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(fiveUTRs_gr)) = c('id','tx_id','gene_id','symbol','type')

    }

    if(any(grepl('3UTR', annotations))) {
        message('Building 3UTRs...')
        ### threeUTRs
            threeUTRs_grl = GenomicFeatures::threeUTRsByTranscript(txdb, use.names = TRUE)
            # Create Rle of the tx_names
            threeUTRs_txname_rle = S4Vectors::Rle(names(threeUTRs_grl), S4Vectors::elementNROWS(threeUTRs_grl))
            threeUTRs_txname_vec = as.character(threeUTRs_txname_rle)
            # Unlist and add the tx_names
            threeUTRs_gr = unlist(threeUTRs_grl, use.names = FALSE)
            GenomicRanges::mcols(threeUTRs_gr)$tx_name = threeUTRs_txname_vec
            # NOTE: here we match on the tx_name because the tx_id is not given
            GenomicRanges::mcols(threeUTRs_gr)$gene_id = id_maps[match(GenomicRanges::mcols(threeUTRs_gr)$tx_name, id_maps$TXNAME), 'GENEID']
            GenomicRanges::mcols(threeUTRs_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(threeUTRs_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(threeUTRs_gr)$type = sprintf('%s_genes_3UTRs', genome)
            GenomicRanges::mcols(threeUTRs_gr)$id = paste0('3UTR:', seq_along(threeUTRs_gr))

            GenomicRanges::mcols(threeUTRs_gr) = GenomicRanges::mcols(threeUTRs_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(threeUTRs_gr)) = c('id','tx_id','gene_id','symbol','type')
    }

    if(any(grepl('exon', annotations)) || any(grepl('intron', annotations))) {

        message('Building exons...')
        ### exons
            exons_grl = GenomicFeatures::exonsBy(txdb, by = 'tx', use.names = TRUE)
            # Create Rle of the tx_names
            exons_txname_rle = S4Vectors::Rle(names(exons_grl), S4Vectors::elementNROWS(exons_grl))
            exons_txname_vec = as.character(exons_txname_rle)
            # Unlist and add the tx_names
            exons_gr = unlist(exons_grl, use.names = FALSE)
            GenomicRanges::mcols(exons_gr)$tx_name = exons_txname_vec
            # Add Entrez ID, symbol, and type
            GenomicRanges::mcols(exons_gr)$gene_id = id_maps[match(GenomicRanges::mcols(exons_gr)$tx_name, id_maps$TXNAME), 'GENEID']
            GenomicRanges::mcols(exons_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(exons_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(exons_gr)$type = sprintf('%s_genes_exons', genome)
            GenomicRanges::mcols(exons_gr)$id = paste0('exon:', seq_along(exons_gr))

            # This needs to be here before we remove the exon_rank mcol in exons_gr
            if(any(grepl('firstexons', annotations))) {
                message('Building first exons...')
                ### first exons
                    firstexons_gr = exons_gr[sapply(exons_gr$exon_rank, function(er){1 %in% er})]
                    # NOTE: The mcol() contents are CharacterLists and IntegerLists, which requires a different approach from previous
                    GenomicRanges::mcols(firstexons_gr)$type = sprintf('%s_genes_firstexons', genome)
                    GenomicRanges::mcols(firstexons_gr)$id = paste0('firstexon:', seq_along(firstexons_gr))

                    GenomicRanges::mcols(firstexons_gr) = GenomicRanges::mcols(firstexons_gr)[, c('id','tx_name','gene_id','symbol','type')]
                    colnames(GenomicRanges::mcols(firstexons_gr)) = c('id','tx_id','gene_id','symbol','type')
            }

            GenomicRanges::mcols(exons_gr) = GenomicRanges::mcols(exons_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(exons_gr)) = c('id','tx_id','gene_id','symbol','type')

        message('Building introns...')
        ### introns
            introns_grl = GenomicFeatures::intronsByTranscript(txdb, use.names = TRUE)
            # Create Rle of the tx_names
            introns_txname_rle = S4Vectors::Rle(names(introns_grl), S4Vectors::elementNROWS(introns_grl))
            introns_txname_vec = as.character(introns_txname_rle)
            # Unlist and add the tx_names
            introns_gr = unlist(introns_grl, use.names = FALSE)
            GenomicRanges::mcols(introns_gr)$tx_name = introns_txname_vec
            # NOTE: here we match on the tx_name because the tx_id is not given
            GenomicRanges::mcols(introns_gr)$gene_id = id_maps[match(GenomicRanges::mcols(introns_gr)$tx_name, id_maps$TXNAME), 'GENEID']
            GenomicRanges::mcols(introns_gr)$symbol = eg2symbol[match(GenomicRanges::mcols(introns_gr)$gene_id, eg2symbol$gene_id), 'symbol']
            GenomicRanges::mcols(introns_gr)$type = sprintf('%s_genes_introns', genome)
            GenomicRanges::mcols(introns_gr)$id = paste0('intron:', seq_along(introns_gr))

            GenomicRanges::mcols(introns_gr) = GenomicRanges::mcols(introns_gr)[, c('id','tx_name','gene_id','symbol','type')]
            colnames(GenomicRanges::mcols(introns_gr)) = c('id','tx_id','gene_id','symbol','type')

        if(any(grepl('intronexonboundaries', annotations))) {
            message('Building intron exon boundaries...')
            ### Intron/exon boundary
                # Intron to exon transition will be the starts of exons_gr for + strand
                # and ends of exons_gr for - strand
                split_exons = split(exons_gr, GenomicRanges::strand(exons_gr))

                intronexon_plus_gr = GenomicRanges::GRanges(
                    seqnames = seqnames(split_exons[['+']]),
                    ranges = IRanges(start = start(split_exons[['+']]), end = start(split_exons[['+']])),
                    strand = GenomicRanges::strand(split_exons[['+']]))
                GenomicRanges::mcols(intronexon_plus_gr) = GenomicRanges::mcols(split_exons[['+']])

                intronexon_minus_gr = GenomicRanges::GRanges(
                    seqnames = seqnames(split_exons[['-']]),
                    ranges = IRanges(start = end(split_exons[['-']]), end = end(split_exons[['-']])),
                    strand = GenomicRanges::strand(split_exons[['-']]))
                GenomicRanges::mcols(intronexon_minus_gr) = GenomicRanges::mcols(split_exons[['-']])

                intronexon_gr = sort(c(intronexon_plus_gr, intronexon_minus_gr))
                seqinfo(intronexon_gr) = seqinfo(exons_gr)

                # Need to remove those boundaries that are TSSs
                tss_idx = unique(S4Vectors::queryHits(GenomicRanges::findOverlaps(intronexon_gr, tss_gr)))
                intronexon_gr = intronexon_gr[-tss_idx]

                # Expand 200bp up and down
                intronexon_gr = GenomicRanges::flank(intronexon_gr, width = 200, both = TRUE)
                GenomicRanges::mcols(intronexon_gr)$type = sprintf('%s_genes_intronexonboundaries', genome)
                GenomicRanges::mcols(intronexon_gr)$id = paste0('intronexonboundary:', seq_along(intronexon_gr))

                GenomicRanges::mcols(intronexon_gr) = GenomicRanges::mcols(intronexon_gr)[, c('id','tx_id','gene_id','symbol','type')]
        }

        if(any(grepl('exonintronboundaries', annotations))) {
            message('Building exon intron boundaries...')
            ### Exon/intron boundary
                if(!exists('split_exons')) {
                    split_exons = split(exons_gr, GenomicRanges::strand(exons_gr))
                }

                # Exon to intron transition will be the ends of exons_gr for + strand
                # and starts of exons_gr for - strand
                exonintron_plus_gr = GenomicRanges::GRanges(
                    seqnames = seqnames(split_exons[['+']]),
                    ranges = IRanges(start = end(split_exons[['+']]), end = end(split_exons[['+']])),
                    strand = GenomicRanges::strand(split_exons[['+']]))
                GenomicRanges::mcols(exonintron_plus_gr) = GenomicRanges::mcols(split_exons[['+']])

                exonintron_minus_gr = GenomicRanges::GRanges(
                    seqnames = seqnames(split_exons[['-']]),
                    ranges = IRanges(start = start(split_exons[['-']]), end = start(split_exons[['-']])),
                    strand = GenomicRanges::strand(split_exons[['-']]))
                GenomicRanges::mcols(exonintron_minus_gr) = GenomicRanges::mcols(split_exons[['-']])

                exonintron_gr = sort(c(exonintron_plus_gr, exonintron_minus_gr))
                seqinfo(exonintron_gr) = seqinfo(exons_gr)

                # Need to remove those boundaries that are TSSs
                tss_idx = unique(S4Vectors::queryHits(GenomicRanges::findOverlaps(exonintron_gr, tss_gr)))
                exonintron_gr = exonintron_gr[-tss_idx]

                # Expand 200bp up and down
                exonintron_gr = GenomicRanges::flank(exonintron_gr, width = 200, both = TRUE)
                GenomicRanges::mcols(exonintron_gr)$type = sprintf('%s_genes_exonintronboundaries', genome)
                GenomicRanges::mcols(exonintron_gr)$id = paste0('exonintronboundary:', seq_along(exonintron_gr))

                GenomicRanges::mcols(exonintron_gr) = GenomicRanges::mcols(exonintron_gr)[, c('id','tx_id','gene_id','symbol','type')]
        }
    }

    ### Put it all together
    mgets = annot_codes[annot_codes$code %in% annotations, 'var']
    genes = do.call('GRangesList', mget(mgets))
    names(genes) = annotations

    return(genes)
}

#' A helper function to build lncRNA annotations.
#'
#' Using the \code{AnnotationHub} package, retrieve transcript level lncRNA annotations for either human (GRCh38) or mouse (GRCm38). If the genome is 'hg19', use the permalink from GENCODE and \code{rtracklayer::import()} to download and process.
#'
#' @param genome The genome assembly.
#'
#' @return A \code{GRanges} object with \code{id} giving the \code{transcript_type} from the GENCODE file, \code{tx_id} being the Ensembl transcript ID, \code{gene_id} being the Entrez ID coming from a mapping of gene symbol to Entrez ID, \code{symbol} being the gene_name from the GENCODE file, and the \code{type} being \code{[genome]_lncrna_gencode}.
build_lncrna_annots = function(genome = c('hg19','hg38','mm10')) {
    # Ensure valid arguments
    genome = match.arg(genome)

    # Get the org.XX.eg.db mapping from Entrez ID to gene symbol
    # First element returned is package name, second is eg2SYMBOL name, third is egENSEMBLTRANS2EG name
    orgdb_name = get_orgdb_name(genome)
    library(sprintf('org.%s.eg.db', orgdb_name), character.only = TRUE)

    # Get Entrez ID to gene symbol mappings
    x = get(sprintf('org.%s.egSYMBOL', orgdb_name))
    mapped_genes = mappedkeys(x)
    eg2symbol = as.data.frame(x[mapped_genes])

    if(genome == 'hg19') {
        use_ah = FALSE
        con = 'ftp://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_19/gencode.v19.long_noncoding_RNAs.gtf.gz'
    } else if (genome == 'hg38') {
        use_ah = TRUE
        hub_genome = 'GRCh38'
        ID = 'AH75123'
    } else if (genome == 'mm10') {
        use_ah = TRUE
        hub_genome = 'GRCm38'
        ID = 'AH49550'
    }

    if(use_ah) {
        # Create AnnotationHub connection
        ah = AnnotationHub::AnnotationHub()
    }

    # Each annotation should be a GRanges object with the following mcols:
    # id, tx_id, gene_id, symbol, type

    message('Building lncRNA transcripts...')
    ### lncRNA transcripts
        # Get the lncRNAs either with AnnotationHub or rtracklayer::import()
        if(use_ah) {
            lncrna_gr = ah[[ID]]
        } else {
            lncrna_gr = rtracklayer::import(con, genome = genome)
        }
        lncrna_gr = lncrna_gr[lncrna_gr$type == 'transcript']

        # Subset the mcols()
        GenomicRanges::mcols(lncrna_gr) = GenomicRanges::mcols(lncrna_gr)[, c('gene_name','transcript_id','transcript_type')]
        colnames(GenomicRanges::mcols(lncrna_gr)) = c('symbol','tx_id','transcript_type')

        # Give the lncRNAs their ids according to the transcript_type
        lncrna_grl = split(lncrna_gr, GenomicRanges::mcols(lncrna_gr)$transcript_type)
        lncrna_grl = S4Vectors::endoapply(lncrna_grl, function(gr){
            GenomicRanges::mcols(gr)$id = paste0(GenomicRanges::mcols(gr)$transcript_type,':', seq_along(gr))
            return(gr)
        })
        lncrna_gr = unlist(lncrna_grl, use.names=FALSE)
        lncrna_gr = GenomicRanges::sort(lncrna_gr)

        # Get the Entrez Gene IDs from the Gene Symbols
        GenomicRanges::mcols(lncrna_gr)$gene_id = eg2symbol[match(GenomicRanges::mcols(lncrna_gr)$symbol, eg2symbol$symbol), 'gene_id']

        # Give it the correct type
        GenomicRanges::mcols(lncrna_gr)$type = sprintf('%s_lncrna_gencode', genome)

        GenomicRanges::mcols(lncrna_gr) = GenomicRanges::mcols(lncrna_gr)[, c('id','tx_id','gene_id','symbol','type')]

        if(use_ah) {
            Seqinfo::seqinfo(lncrna_gr) = Seqinfo::Seqinfo(genome = genome)
        }

    return(lncrna_gr)
}

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("annotatr")

## ----eval=FALSE---------------------------------------------------------------
# devtools::install_github('rcavalcante/annotatr')

## ----echo=FALSE---------------------------------------------------------------
suppressWarnings(suppressMessages(suppressPackageStartupMessages(library(annotatr))))

## ----warning = FALSE, message = FALSE-----------------------------------------
# Create a named vector for the AnnotationHub accession codes with desired names
h3k4me3_codes = c('Gm12878' = 'AH23256')
# Fetch ah_codes from AnnotationHub and create annotations annotatr understands
build_ah_annots(genome = 'hg19', ah_codes = h3k4me3_codes, annotation_class = 'H3K4me3')
# The annotations as they appear in annotatr_cache
ah_names = c('hg19_H3K4me3_Gm12878')

print(annotatr_cache$get('hg19_H3K4me3_Gm12878'))

## ----warning = FALSE, message = FALSE-----------------------------------------
## Use ENCODE ChIP-seq peaks for EZH2 in GM12878
## These files contain chr, start, and end columns
ezh2_file = system.file('extdata', 'Gm12878_Ezh2_peak_annotations.txt.gz', package = 'annotatr')

## Custom annotation objects are given names of the form genome_custom_name
read_annotations(con = ezh2_file, genome = 'hg19', name = 'ezh2', format = 'bed')

print(annotatr_cache$get('hg19_custom_ezh2'))

## ----warning = FALSE, message = FALSE-----------------------------------------
print(annotatr_cache$list_env())

## ----warning = FALSE, message = FALSE-----------------------------------------
# This file in inst/extdata represents regions tested for differential
# methylation between two conditions. Additionally, there are columns
# reporting the p-value on the test for differential meth., the
# meth. difference between the two groups, and the group meth. rates.
dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
extraCols = c(diff_meth = 'numeric', mu0 = 'numeric', mu1 = 'numeric')
dm_regions = read_regions(con = dm_file, genome = 'hg19', extraCols = extraCols, format = 'bed',
    rename_name = 'DM_status', rename_score = 'pval')
# Use less regions to speed things up
dm_regions = dm_regions[1:2000]
print(dm_regions)

## ----warning = FALSE, message = FALSE-----------------------------------------
# Select annotations for intersection with regions
# Note inclusion of custom annotation, and use of shortcuts
annots = c('hg19_cpgs', 'hg19_basicgenes', 'hg19_genes_intergenic',
    'hg19_genes_intronexonboundaries',
    'hg19_custom_ezh2', 'hg19_H3K4me3_Gm12878')

# Build the annotations (a single GRanges object)
annotations = build_annotations(genome = 'hg19', annotations = annots)

# Intersect the regions we read in with the annotations
dm_annotated = annotate_regions(
    regions = dm_regions,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = FALSE)
# A GRanges object is returned
print(dm_annotated)

## ----warning = FALSE, message = FALSE-----------------------------------------
# Coerce to a data.frame
df_dm_annotated = data.frame(dm_annotated)

# See the GRanges column of dm_annotaed expanded
print(head(df_dm_annotated))

# Subset based on a gene symbol, in this case NOTCH1
notch1_subset = subset(df_dm_annotated, annot.symbol == 'NOTCH1')
print(head(notch1_subset))

## ----warning = FALSE, message = FALSE-----------------------------------------
# Randomize the input regions
dm_random_regions = randomize_regions(
    regions = dm_regions,
    allow.overlaps = TRUE,
    per.chromosome = TRUE)

# Annotate the random regions using the same annotations as above
# These will be used in later functions
dm_random_annotated = annotate_regions(
    regions = dm_random_regions,
    annotations = annotations,
    ignore.strand = TRUE,
    quiet = TRUE)

## ----warning = FALSE, message = FALSE-----------------------------------------
# Find the number of regions per annotation type
dm_annsum = summarize_annotations(
    annotated_regions = dm_annotated,
    quiet = TRUE)
print(dm_annsum)

# Find the number of regions per annotation type
# and the number of random regions per annotation type
dm_annsum_rnd = summarize_annotations(
    annotated_regions = dm_annotated,
    annotated_random = dm_random_annotated,
    quiet = TRUE)
print(dm_annsum_rnd)

# Take the mean of the diff_meth column across all regions
# occurring in an annotation.
dm_numsum = summarize_numerical(
    annotated_regions = dm_annotated,
    by = c('annot.type', 'annot.id'),
    over = c('diff_meth'),
    quiet = TRUE)
print(dm_numsum)

# Count the occurrences of classifications in the DM_status
# column across the annotation types.
dm_catsum = summarize_categorical(
    annotated_regions = dm_annotated,
    by = c('annot.type', 'DM_status'),
    quiet = TRUE)
print(dm_catsum)

## ----fig.align='center', fig.cap='Number of DM regions per annotation.', fig.height=6, fig.width=6, fig.show = 'hold', warning = FALSE, message = FALSE----
# View the number of regions per annotation. This function
# is useful when there is no classification or data
# associated with the regions.
annots_order = c(
    'hg19_custom_ezh2',
    'hg19_H3K4me3_Gm12878',
    'hg19_genes_1to5kb',
    'hg19_genes_promoters',
    'hg19_genes_5UTRs',
    'hg19_genes_exons',
    'hg19_genes_intronexonboundaries',
    'hg19_genes_introns',
    'hg19_genes_3UTRs',
    'hg19_genes_intergenic')
dm_vs_kg_annotations = plot_annotation(
    annotated_regions = dm_annotated,
    annotation_order = annots_order,
    plot_title = '# of Sites Tested for DM annotated on chr9',
    x_label = 'knownGene Annotations',
    y_label = 'Count')
print(dm_vs_kg_annotations)

## ----fig.align='center', fig.cap='Number of DM regions per annotation with randomized regions.', fig.height=6, fig.width=6, fig.show = 'hold', warning = FALSE, message = FALSE----
# View the number of regions per annotation and include the annotation
# of randomized regions
annots_order = c(
    'hg19_custom_ezh2',
    'hg19_H3K4me3_Gm12878',
    'hg19_genes_1to5kb',
    'hg19_genes_promoters',
    'hg19_genes_5UTRs',
    'hg19_genes_exons',
    'hg19_genes_intronexonboundaries',
    'hg19_genes_introns',
    'hg19_genes_3UTRs',
    'hg19_genes_intergenic')
dm_vs_kg_annotations_wrandom = plot_annotation(
    annotated_regions = dm_annotated,
    annotated_random = dm_random_annotated,
    annotation_order = annots_order,
    plot_title = 'Dist. of Sites Tested for DM (with rndm.)',
    x_label = 'Annotations',
    y_label = 'Count')
print(dm_vs_kg_annotations_wrandom)

## ----fig.align='center', fig.cap='Number of DM regions per pair of annotations.', fig.height=8, fig.width=8, fig.show = 'hold', warning = FALSE, message = FALSE----
# View a heatmap of regions occurring in pairs of annotations
annots_order = c(
    'hg19_custom_ezh2',
    'hg19_H3K4me3_Gm12878',
    'hg19_genes_promoters',
    'hg19_genes_5UTRs',
    'hg19_genes_exons',
    'hg19_genes_introns',
    'hg19_genes_3UTRs',
    'hg19_genes_intergenic')
dm_vs_coannotations = plot_coannotations(
    annotated_regions = dm_annotated,
    annotation_order = annots_order,
    axes_label = 'Annotations',
    plot_title = 'Regions in Pairs of Annotations')
print(dm_vs_coannotations)

## ----fig.align='center', fig.cap='Methylation Rates in Group 0 for Regions Over DM Status.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
dm_vs_regions_annot = plot_numerical(
    annotated_regions = dm_annotated,
    x = 'mu0',
    facet = 'annot.type',
    facet_order = c('hg19_genes_1to5kb','hg19_genes_promoters',
        'hg19_genes_5UTRs','hg19_genes_3UTRs', 'hg19_custom_ezh2',
        'hg19_genes_intergenic', 'hg19_cpg_islands'),
    bin_width = 5,
    plot_title = 'Group 0 Region Methylation In Genes',
    x_label = 'Group 0')
print(dm_vs_regions_annot)

## ----fig.align='center', fig.cap='Methylation Differences for Regions Over DM Status and Annotation Type.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
dm_vs_regions_annot2 = plot_numerical(
    annotated_regions = dm_annotated,
    x = 'diff_meth',
    facet = c('annot.type','DM_status'),
    facet_order = list(c('hg19_genes_promoters','hg19_genes_5UTRs','hg19_cpg_islands'), c('hyper','hypo','none')),
    bin_width = 5,
    plot_title = 'Group 0 Region Methylation In Genes',
    x_label = 'Methylation Difference')
print(dm_vs_regions_annot2)

## ----fig.align='center', fig.cap='Methylation Rates in Regions Over DM Status in Group 0 vs Group 1.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
dm_vs_regions_name = plot_numerical(
    annotated_regions = dm_annotated,
    x = 'mu0',
    y = 'mu1',
    facet = 'annot.type',
    facet_order = c('hg19_genes_1to5kb','hg19_genes_promoters',
        'hg19_genes_5UTRs','hg19_genes_3UTRs', 'hg19_custom_ezh2',
        'hg19_genes_intergenic', 'hg19_cpg_islands', 'hg19_cpg_shores'),
    plot_title = 'Region Methylation: Group 0 vs Group 1',
    x_label = 'Group 0',
    y_label = 'Group 1')
print(dm_vs_regions_name)

## ----fig.align='center', fig.cap='Group 0 methylation Rates in Regions in promoters, CpG islands, and both.', fig.height=5, fig.width=12, fig.show='hold', warning = FALSE, message = FALSE----
dm_vs_num_co = plot_numerical_coannotations(
    annotated_regions = dm_annotated,
    x = 'mu0',
    annot1 = 'hg19_cpg_islands',
    annot2 = 'hg19_genes_promoters',
    bin_width = 5,
    plot_title = 'Group 0 Perc. Meth. in CpG Islands and Promoters',
    x_label = 'Percent Methylation')
print(dm_vs_num_co)

## ----fig.align='center', fig.cap='Differential methylation classification with counts of CpG annotations.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
# View the counts of CpG annotations in data classes

# The orders for the x-axis labels. This is also a subset
# of the labels (hyper, hypo, none).
x_order = c(
    'hyper',
    'hypo')
# The orders for the fill labels. Can also use this
# parameter to subset annotation types to fill.
fill_order = c(
    'hg19_cpg_islands',
    'hg19_cpg_shores',
    'hg19_cpg_shelves',
    'hg19_cpg_inter')
# Make a barplot of the data class where each bar
# is composed of the counts of CpG annotations.
dm_vs_cpg_cat1 = plot_categorical(
    annotated_regions = dm_annotated, x='DM_status', fill='annot.type',
    x_order = x_order, fill_order = fill_order, position='stack',
    plot_title = 'DM Status by CpG Annotation Counts',
    legend_title = 'Annotations',
    x_label = 'DM status',
    y_label = 'Count')
print(dm_vs_cpg_cat1)

## ----fig.align='center', fig.cap='Differential methylation classification with proportion of CpG annotations.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
# Use the same order vectors as the previous code block,
# but use proportional fill instead of counts.

# Make a barplot of the data class where each bar
# is composed of the *proportion* of CpG annotations.
dm_vs_cpg_cat2 = plot_categorical(
    annotated_regions = dm_annotated, x='DM_status', fill='annot.type',
    x_order = x_order, fill_order = fill_order, position='fill',
    plot_title = 'DM Status by CpG Annotation Proportions',
    legend_title = 'Annotations',
    x_label = 'DM status',
    y_label = 'Proportion')
print(dm_vs_cpg_cat2)

## ----fig.align='center', fig.cap='Differential methylation classification with proportion of CpG annotations and random regions.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
# Add in the randomized annotations for "Random Regions" bar

# Make a barplot of the data class where each bar
# is composed of the *proportion* of CpG annotations, and
# includes "All" regions tested for DM and "Random Regions"
# regions consisting of randomized regions.
dm_vs_cpg_cat_random = plot_categorical(
    annotated_regions = dm_annotated, annotated_random = dm_random_annotated,
    x='DM_status', fill='annot.type',
    x_order = x_order, fill_order = fill_order, position='fill',
    plot_title = 'DM Status by CpG Annotation Proportions',
    legend_title = 'Annotations',
    x_label = 'DM status',
    y_label = 'Proportion')
print(dm_vs_cpg_cat_random)

## ----fig.align='center', fig.cap='Basic gene annotations with proportions of DM classification.', fig.height=6, fig.width=6, fig.show='hold', warning = FALSE, message = FALSE----
# View the proportions of data classes in knownGene annotations

# The orders for the x-axis labels.
x_order = c(
    'hg19_custom_ezh2',
    'hg19_genes_1to5kb',
    'hg19_genes_promoters',
    'hg19_genes_5UTRs',
    'hg19_genes_exons',
    'hg19_genes_introns',
    'hg19_genes_3UTRs',
    'hg19_genes_intergenic')
# The orders for the fill labels.
fill_order = c(
    'hyper',
    'hypo',
    'none')
dm_vs_kg_cat = plot_categorical(
    annotated_regions = dm_annotated, x='annot.type', fill='DM_status',
    x_order = x_order, fill_order = fill_order, position='fill',
    legend_title = 'DM Status',
    x_label = 'knownGene Annotations',
    y_label = 'Proportion')
print(dm_vs_kg_cat)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarize_annotations}
\alias{summarize_annotations}
\title{Summarize annotation counts}
\usage{
summarize_annotations(annotated_regions, annotated_random, quiet = FALSE)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{annotated_random}{The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{tbl_df} of the number of regions per annotation type.
}
\description{
Given a \code{GRanges} of annotated regions, count the number of regions in each annotation type. If \code{annotated_random} is not \code{NULL}, then the same is computed for the random regions.
}
\details{
If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once. For example, if a region were annotated to multiple exons, it would only count once toward the exons, but if it were annotated to an exon and an intron, it would count towards both.
}
\examples{
   ### An example of ChIP-seq peaks with signalValue

   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
   r = read_regions(con = file, genome = 'hg19')

   a = annotate_regions(
       regions = r,
       annotations = annotations,
       ignore.strand = TRUE,
       quiet = FALSE)

   rnd = randomize_regions(regions = r)

   rnd_annots = annotate_regions(
       regions = rnd,
       annotations = annotations,
       ignore.strand = TRUE,
       quiet = FALSE)

   # Summarize the annotated regions without randomized regions
   s = summarize_annotations(annotated_regions = a)

   # Summarize the annotated regions with randomized regions
   s_rnd = summarize_annotations(
       annotated_regions = a,
       annotated_random = rnd_annots)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{plot_numerical_coannotations}
\alias{plot_numerical_coannotations}
\title{Plot numerical data occurring in pairs of annotations}
\usage{
plot_numerical_coannotations(
  annotated_regions,
  x,
  y,
  annot1,
  annot2,
  bin_width = 10,
  plot_title,
  x_label,
  y_label,
  legend_facet_label,
  legend_cum_label,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{A \code{GRanges} returned from \code{annotate_regions()}.}

\item{x}{A string indicating the column of the \code{GRanges} to use for the x-axis.}

\item{y}{A string indicating the column of the \code{GRanges} to use for the y-axis. If missing, a histogram over \code{x} will be plotted. If not missing, a scatterplot is plotted.}

\item{annot1}{A string indicating the first annotation type.}

\item{annot2}{A string indicating the second annotation type.}

\item{bin_width}{An integer indicating the bin width of the histogram used for score. Default 10. Select something appropriate for the data. NOTE: This is only used if \code{y} is \code{NULL}.}

\item{plot_title}{A string used for the title of the plot. If missing, no title is displayed.}

\item{x_label}{A string used for the x-axis label. If missing, no x-axis label is displayed.}

\item{y_label}{A string used for the y-axis label. If missing, no y-axis label is displayed.}

\item{legend_facet_label}{A string used to label the gray bar portion of the legend. Defaults to "x in annot pair".}

\item{legend_cum_label}{A string used to label the red outline portion of the legend. Defaults to "All x".}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
}
\description{
Plot numerical data associated with regions occurring in \code{annot1}, \code{annot2} and in both. As with \code{plot_numerical()}, the result is a plot of histograms or x-y scatterplots.
}
\details{
For example, a CpG with associated percent methylation annotated to a CpG island and a promoter will count once in the All distribution and once in the CpG island / promoter facet distribution. However, a CpG associated only with a promoter will count once in the All distribution and once in the promoter / promoter distribution.
}
\examples{
   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
   extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
   dm_regions = read_regions(con = dm_file, extraCols = extraCols,
       rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
   dm_regions = dm_regions[1:1000]

   dm_annots = annotate_regions(
       regions = dm_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   dm_vs_num_co = plot_numerical_coannotations(
       annotated_regions = dm_annots,
       x = 'mu0',
       annot1 = 'hg19_cpg_islands',
       annot2 = 'hg19_cpg_shelves',
       bin_width = 5,
       plot_title = 'Group 0 Perc. Meth. in CpG Islands and Promoters',
       x_label = 'Percent Methylation')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_annotations}
\alias{build_annotations}
\title{A function to build annotations from TxDb.* and AnnotationHub resources}
\usage{
build_annotations(genome, annotations)
}
\arguments{
\item{genome}{The genome assembly.}

\item{annotations}{A character vector of annotations to build. Valid annotation codes are listed with \code{builtin_annotations()}. The "basicgenes" shortcut builds the following regions: 1-5Kb upstream of TSSs, promoters, 5UTRs, exons, introns, and 3UTRs. The "cpgs" shortcut builds the following regions: CpG islands, shores, shelves, and interCGI regions. NOTE: Shortcuts need to be appended by the genome, e.g. \code{hg19_basicgenes}.
Custom annotations whose names are of the form \code{[genome]_custom_[name]} should also be included. Custom annotations should be read in and converted to \code{GRanges} with \code{read_annotations()}. They can be for a \code{supported_genome()}, or for an unsupported genome.}
}
\value{
A \code{GRanges} object of all the \code{annotations} combined. The \code{mcols} are \code{id, tx_id, gene_id, symbol, type}. The \code{id} column is a unique name, the \code{tx_id} column is either a UCSC knownGene transcript ID (genic annotations) or a Ensembl transcript ID (lncRNA annotations), the \code{gene_id} is the Entrez ID, the \code{symbol} is the gene symbol from the \code{org.*.eg.db} mapping from the Entrez ID, and the \code{type} is of the form \code{[genome]_[type]_[name]}.
}
\description{
Create a \code{GRanges} object consisting of all the desired \code{annotations}. Supported annotation codes are listed by \code{builtin_annotations()}. The basis for enhancer annotations are FANTOM5 data, the basis for CpG related annotations are CpG island tracks from \code{AnnotationHub}, and the basis for genic annotations are from the \code{TxDb.*} and \code{org.db} group of packages.
}
\examples{
# Example with hg19 gene promoters
annots = c('hg19_genes_promoters')
annots_gr = build_annotations(genome = 'hg19', annotations = annots)

# See vignette for an example with custom annotation

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_gene_annots}
\alias{build_gene_annots}
\title{A helper function to build genic annotations.}
\usage{
build_gene_annots(
  genome = annotatr::builtin_genomes(),
  annotations = annotatr::builtin_annotations()
)
}
\arguments{
\item{genome}{The genome assembly.}

\item{annotations}{A character vector with entries of the form \code{[genome]_genes_{1to5kb,promoters,5UTRs,cds,exons,firstexons,introns,intronexonboundaries,exonintronboundaries,3UTRs,intergenic}}.}
}
\value{
A list of \code{GRanges} objects with unique \code{id} of the form \code{[type]:i}, \code{tx_id} being the UCSC knownGene transcript name, \code{gene_id} being the Entrez Gene ID, \code{symbol} being the gene symbol from the Entrez ID to symbol mapping in \code{org.db} for that species, and \code{type} being the annotation type.
}
\description{
Using the \code{TxDb.*} group of packages, construct genic annotations consisting of any combination of 1-5kb upstream of a TSS, promoters (< 1kb from TSS), 5UTRs, CDS, exons, first exons, introns, intron/exon and exon/intron boundaries, 3UTRs, and intergenic.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randomize.R
\name{randomize_regions}
\alias{randomize_regions}
\title{Randomize Regions}
\usage{
randomize_regions(
  regions,
  allow.overlaps = TRUE,
  per.chromosome = TRUE,
  quiet = FALSE
)
}
\arguments{
\item{regions}{A \code{GRanges} object from \code{read_regions}.}

\item{allow.overlaps}{A logical stating whether random regions can overlap input regions (TRUE) or not (FALSE). Default TRUE.}

\item{per.chromosome}{A logical stating whether the random regions should remain on the same chromosome (TRUE) or not (FALSE). Default TRUE.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{GRanges} object of randomized regions based on \code{regions} from \code{read_regions()}. NOTE: Data associated with the original regions is not attached to the randomized regions.
}
\description{
\code{randomize_regions} is a wrapper function for \code{regioneR::randomizeRegions()} that simplifies the creation of randomized regions for an input set of regions read with \code{read_regions()}. It relies on the \code{seqlengths} of \code{regions} in order to build the appropriate \code{genome} object for \code{regioneR::randomizeRegions()}.
}
\details{
NOTE: The data associated with the input \code{regions} are not passed on to the random regions.
}
\examples{
   # Create random region set based on ENCODE ChIP-seq data
   file = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
   r = read_regions(con = file, genome = 'hg19')

   random_r = randomize_regions(regions = r)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_orgdb_name}
\alias{get_orgdb_name}
\title{Function to get correct org.* package name based on genome}
\usage{
get_orgdb_name(genome = annotatr::builtin_genomes())
}
\arguments{
\item{genome}{A string giving the genome assembly.}
}
\value{
A string giving the correct org for org.db packages. e.g. hg19 -> Hs.
}
\description{
Function to get correct org.* package name based on genome
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tidy_annotations}
\alias{tidy_annotations}
\title{Function to tidy up annotation accessors for visualization}
\usage{
tidy_annotations(annotations)
}
\arguments{
\item{annotations}{A character vector of annotations, in the order they are to appear in the visualization.}
}
\value{
A list of mappings from original annotation names to names ready for visualization.
}
\description{
Function to tidy up annotation accessors for visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_hmm_annots}
\alias{build_hmm_annots}
\title{A helper function to build chromHMM annotations for hg19 from UCSC Genome Browser.}
\usage{
build_hmm_annots(
  genome = c("hg19"),
  annotations = annotatr::builtin_annotations()
)
}
\arguments{
\item{genome}{The genome assembly.}

\item{annotations}{A character vector of valid chromatin state annotatin codes.}
}
\value{
A \code{GRanges} object.
}
\description{
A helper function to build chromHMM annotations for hg19 from UCSC Genome Browser.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{reformat_hmm_codes}
\alias{reformat_hmm_codes}
\title{Function to recode classes from chromHMM type column}
\usage{
reformat_hmm_codes(hmm_codes)
}
\arguments{
\item{hmm_codes}{in the original form from UCSC Genome Browser track.}
}
\value{
A character vector of chromHMM classes with numbers and underscores removed.
}
\description{
Function to recode classes from chromHMM type column
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{builtin_annotations}
\alias{builtin_annotations}
\title{Function listing which annotations are available.}
\usage{
builtin_annotations()
}
\value{
A character vector of available annotations.
}
\description{
This includes the shortcuts. The \code{expand_annotations()} function helps
handle the shortcuts.
}
\examples{
builtin_annotations()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{read_regions}
\alias{read_regions}
\title{Read genomic regions in BEDX+Y format}
\usage{
read_regions(
  con,
  genome = NA,
  format,
  extraCols = character(),
  rename_name,
  rename_score,
  ...
)
}
\arguments{
\item{con}{A path, URL, connection or BEDFile object. See \code{rtracklayer::import()} documentation.}

\item{genome}{From \code{rtracklayer::import()}: The identifier of a genome, or NA if unknown. Typically, this is a UCSC identifier like 'hg19'. An attempt will be made to derive the \code{seqinfo} on the return value using either an installed BSgenome package or UCSC, if network access is available.}

\item{format}{From \code{rtracklayer::import()}: The format of the output. If not missing, should be one of 'bed', 'bed15', 'bedGraph' or 'bedpe'. If missing and 'con' is a filename, the format is derived from the file extension. This argument is unnecessary when 'con' is a derivative of 'RTLFile'.}

\item{extraCols}{From \code{rtracklayer::import()}: A character vector in the same form as 'colClasses' from 'read.table'.  It should indicate the name and class of each extra/special column to read from the BED file. As BED does not encode column names, these are assumed to be the last columns in the file. This enables parsing of the various BEDX+Y formats.}

\item{rename_name}{A string to rename the name column of the BED file. For example, if the name column actually contains a categorical variable.}

\item{rename_score}{A string to rename the score column of the BED file. For example, if the score column represents a quantity about the data besides the score in the BED specification.}

\item{...}{Parameters to pass onto the format-specific method of \code{rtracklayer::import()}.}
}
\value{
A \code{GRanges} object.
}
\description{
\code{read_regions()} reads genomic regions by calling the \code{rtracklayer::import()} function. This function can automatically deal with BEDX files from BED3 to BED6. For BED6+Y, the \code{extraCols} argument should be used to correctly interpret the extra columns.
}
\details{
NOTE: The \code{name} (4th) and \code{score} (5th) columns are so named. If these columns have a particular meaning for your data, they should be renamed with the \code{rename_name} and/or \code{rename_score} parameters.
}
\examples{

   # Example of reading a BED6+3 file where the last 3 columns are non-standard
   file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
   extraCols = c(diff_meth = 'numeric', mu0 = 'numeric', mu1 = 'numeric')
   gr = read_regions(con = file, genome = 'hg19', extraCols = extraCols, format = 'bed',
       rename_name = 'DM_status', rename_score = 'pval')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_ah_annots}
\alias{build_ah_annots}
\title{A helper function to build arbitrary annotatinos from AnnotationHub}
\usage{
build_ah_annots(genome, ah_codes, annotation_class)
}
\arguments{
\item{genome}{The genome assembly.}

\item{ah_codes}{A named character vector giving the AnnotationHub accession number (e.g. AH23256), and whose name describes what the annotation is (e.g. Gm12878_H3K4me3).}

\item{annotation_class}{A string to name the group of annotations in \code{ah_codes}}
}
\value{
A \code{GRanges} object stored in \code{annotatr_cache}. To view an annotation built with this function, do \code{annotatr_cache$get(name)}. To add these annotations to a set of annotations, include \code{'[genome]_[annotation_class]_[name]'} in the call to \code{build_annotations()}. See example below.
}
\description{
A helper function to build arbitrary annotatinos from AnnotationHub
}
\examples{

# Create a named vector for the AnnotationHub accession codes with desired names
h3k4me3_code = c('Gm12878' = 'AH23256')
# Fetch ah_codes from AnnotationHub and create annotations annotatr understands
build_ah_annots(genome = 'hg19', ah_codes = h3k4me3_code, annotation_class = 'H3K4me3')
# The annotations as they appear in annotatr_cache
annot_name = c('hg19_H3K4me3_Gm12878')
# Build the annotations right before annotating any regions
annotations = build_annotations(genome = 'hg19', annotations = annot_name)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{subset_order_tbl}
\alias{subset_order_tbl}
\title{Function to subset a tbl_df or grouped_df by a column}
\usage{
subset_order_tbl(tbl, col, col_order)
}
\arguments{
\item{tbl}{A \code{tbl_df} or \code{grouped_df}.}

\item{col}{A string indicating which column of of \code{tbl} to subset and order}

\item{col_order}{A character vector indicating the order of \code{col}.}
}
\value{
A modified version of \code{summary} with \code{col} subsetted by \code{col_order}.
}
\description{
Function to subset a tbl_df or grouped_df by a column
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{expand_annotations}
\alias{expand_annotations}
\title{Function to expand annotation shortcuts}
\usage{
expand_annotations(annotations)
}
\arguments{
\item{annotations}{A character vector of annotations, possibly using the shortcut accessors}
}
\value{
A vector of data accession-ized names that are ordered from upstream to downstream in the case of knownGenes and islands to interCGI in the case of cpgs.
}
\description{
Function to expand annotation shortcuts
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotatr_data_doc.R
\docType{data}
\name{annotations}
\alias{annotations}
\title{example_annotations data}
\format{
A \code{GRanges} object with the CpG feature annotations for hg19
and containing \code{mcols}:
\describe{
    \item{id}{The internal ID for the annotation}
    \item{tx_id}{All NA, since these are not associated with tx_ids}
    \item{gene_id}{All NA, since there are not associated Entrez IDs}
    \item{symbols}{All NA, since there are not associated gene symbols}
    \item{type}{A character indicating the type of annotation. Including:
'hg19_cpg_islands', 'hg19_cpg_shores', 'hg19_cpg_shelves', and 'hg19_cpg_inter'.}
}
}
\source{
The AnnotationHub resource for hg19 CpG features.
}
\usage{
annotations
}
\description{
A \code{GRanges} of precomputed annotations for CpG features. Created by doing
\code{build_annotations(genome='hg19', annotations = 'hg19_cpgs')}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersect.R
\name{annotate_regions}
\alias{annotate_regions}
\title{A function to intersect user region data with annotation data}
\usage{
annotate_regions(
  regions,
  annotations,
  minoverlap = 1L,
  ignore.strand = TRUE,
  quiet = FALSE
)
}
\arguments{
\item{regions}{The GRanges object returned by \code{read_regions()}.}

\item{annotations}{A character vector of annotations to build. Valid annotation codes are listed with \code{builtin_annotations()}. The "basicgenes" shortcut builds the following regions: 1-5Kb upstream of TSSs, promoters, 5UTRs, exons, introns, and 3UTRs. The "cpgs" shortcut builds the following regions: CpG islands, shores, shelves, and interCGI regions. NOTE: Shortcuts need to be appended by the genome, e.g. \code{hg19_basicgenes}.
Custom annotations whose names are of the form \code{[genome]_custom_[name]} should also be included. Custom annotations should be read in and converted to \code{GRanges} with \code{read_annotations()}. They can be for a \code{supported_genome()}, or for an unsupported genome.}

\item{minoverlap}{A scalar, positive integer, indicating the minimum required overlap of regions with annotations.}

\item{ignore.strand}{Logical indicating whether strandedness should be respected in findOverlaps(). Default FALSE.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{GRanges} where the \code{granges} are from the regions, and the \code{mcols} include the \code{mcols} from the regions and a column with the annotation \code{GRanges}.
}
\description{
Annotate genomic regions to selected genomic annotations while preserving the data associated with the genomic regions.
}
\examples{
   r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
   extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
   r = read_regions(con = r_file, extraCols = extraCols, rename_score = 'coverage')

   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   a = annotate_regions(
       regions = r,
       annotations = annotations,
       ignore.strand = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{plot_annotation}
\alias{plot_annotation}
\title{Plot the number of regions per annotation}
\usage{
plot_annotation(
  annotated_regions,
  annotated_random,
  annotation_order = NULL,
  plot_title,
  x_label,
  y_label,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{annotated_random}{The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}.}

\item{annotation_order}{A character vector which doubles as the subset of annotations desired for the plot as well as the ordering. If \code{NULL}, all annotations are displayed.}

\item{plot_title}{A string used for the title of the plot. If missing, no title is displayed.}

\item{x_label}{A string used for the x-axis label. If missing, no x-axis label is displayed.}

\item{y_label}{A string used for the y-axis label. If missing, no y-axis label is displayed.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{ggplot} object which can be viewed by calling it, saved with \code{ggplot2::ggsave}, or edited.
}
\description{
Given a \code{GRanges} of annotated regions, plot the number of regions with the corresponding genomic annotations used in \code{annotation_order}. If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once in the corresponding bar plot. For example, if a region were annotated to multiple exons, it would only count once toward the exon bar in the plot, but if it were annotated to an exon and an intron, it would count towards both.
}
\examples{
   ########################################################################
   # An example of ChIP-seq peaks with signalValue used for score

   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   chip_bed = system.file('extdata', 'Gm12878_Stat3_chr2.bed.gz', package = 'annotatr')
   chip_regions = read_regions(con = chip_bed, genome = 'hg19')

   chip_rnd = randomize_regions(regions = chip_regions)

   chip_annots = annotate_regions(
       regions = chip_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   chip_rnd_annots = annotate_regions(
       regions = chip_rnd,
       annotations = annotations,
       ignore.strand = TRUE)

   annots_order = c(
       'hg19_cpg_islands',
       'hg19_cpg_shores')

   p_annots = plot_annotation(annotated_regions = chip_annots,
       annotation_order = annots_order)
   p_annots_rnd = plot_annotation(annotated_regions = chip_annots,
       annotated_random = chip_rnd_annots, annotation_order = annots_order)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{plot_categorical}
\alias{plot_categorical}
\title{Plot a categorical data variable over another}
\usage{
plot_categorical(
  annotated_regions,
  annotated_random,
  x,
  fill = NULL,
  x_order = NULL,
  fill_order = NULL,
  position = "stack",
  plot_title,
  legend_title,
  x_label,
  y_label,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{annotated_random}{The \code{GRanges} result of \code{annotate_regions()} on the randomized regions created from \code{randomize_regions()}. Random regions can only be used with \code{fill == 'annot.type'}.}

\item{x}{One of 'annot.type' or a categorical data column, indicating whether annotation classes or data classes will appear on the x-axis.}

\item{fill}{One of 'annot.type', a categorical data column, or \code{NULL}, indicating whether annotation classes or data classes will fill the bars. If \code{NULL} then the bars will be the total counts of the x classes.}

\item{x_order}{A character vector that subsets and orders the x classes. Default \code{NULL}, uses existing values.}

\item{fill_order}{A character vector that subsets and orders the fill classes. Default \code{NULL}, uses existing values.}

\item{position}{A string which has the same possible values as in \code{ggplot2::geom_bar(..., position)}, i.e., 'stack', 'fill', 'dodge', etc.}

\item{plot_title}{A string used for the title of the plot. If missing, no title is displayed.}

\item{legend_title}{A string used for the legend title to describe fills (if fill is not \code{NULL}). Default displays corresponding variable name.}

\item{x_label}{A string used for the x-axis label. If missing, corresponding variable name used.}

\item{y_label}{A string used for the y-axis label. If missing, corresponding variable name used.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
}
\description{
Given a \code{GRanges} of annotated regions from \code{annotate_regions()}, visualize the the distribution of categorical data \code{fill} in categorical data \code{x}. A bar representing the distribution of all \code{fill} in \code{x} will be added according to the contents of \code{fill}. This is the distribution over all values of \code{x}. Additionally, when \code{annotated_random} is not missing, a "Random Regions" bar shows the distribution of random regions over \code{fill}.
}
\details{
For example, if a differentially methylated region has the categorical label hyper, and is annotated to a promoter, a 5UTR, two exons, and an intron. Each annotation will appear in the All bar once. Likewise for the hyper bar if the differential methylation status is chosen as \code{x} with \code{annot.type} chosen as \code{fill}.
}
\examples{
   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
   extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
   dm_regions = read_regions(con = dm_file, extraCols = extraCols, genome = 'hg19',
       rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
   dm_regions = dm_regions[1:1000]

   dm_annots = annotate_regions(
       regions = dm_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   dm_order = c(
       'hyper',
       'hypo')
   cpg_order = c(
       'hg19_cpg_islands',
       'hg19_cpg_shores',
       'hg19_cpg_shelves',
       'hg19_cpg_inter')

   dm_vn = plot_categorical(
       annotated_regions = dm_annots,
       x = 'DM_status',
       fill = 'annot.type',
       x_order = dm_order,
       fill_order = cpg_order,
       position = 'fill',
       legend_title = 'knownGene Annotations',
       x_label = 'DM status',
       y_label = 'Proportion')

   # Create randomized regions
   dm_rnd_regions = randomize_regions(regions = dm_regions)
   dm_rnd_annots = annotate_regions(
       regions = dm_rnd_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   dm_vn_rnd = plot_categorical(
       annotated_regions = dm_annots,
       annotated_random = dm_rnd_annots,
       x = 'DM_status',
       fill = 'annot.type',
       x_order = dm_order,
       fill_order = cpg_order,
       position = 'fill',
       legend_title = 'knownGene Annotations',
       x_label = 'DM status',
       y_label = 'Proportion')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_enhancer_annots}
\alias{build_enhancer_annots}
\title{A helper function to build enhancer annotations for hg19 and mm10 from FANTOM5.}
\usage{
build_enhancer_annots(genome = c("hg19", "hg38", "mm9", "mm10"))
}
\arguments{
\item{genome}{The genome assembly.}
}
\value{
A \code{GRanges} object.
}
\description{
A helper function to build enhancer annotations for hg19 and mm10 from FANTOM5.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_annotations}
\alias{check_annotations}
\title{Function to check for valid annotations}
\usage{
check_annotations(annotations)
}
\arguments{
\item{annotations}{A character vector of annotations possibly using the shortcuts}
}
\value{
If all the checks on the annotations pass, returns NULL to allow code to move forward.
}
\description{
Gives errors if any annotations are not in builtin_annotations() (and they are not in the required custom format), basicgenes are used, or the genome prefixes are not the same for all annotations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{plot_numerical}
\alias{plot_numerical}
\title{Plot numerical data over regions or regions summarized over annotations}
\usage{
plot_numerical(
  annotated_regions,
  x,
  y,
  facet,
  facet_order,
  bin_width = 10,
  plot_title,
  x_label,
  y_label,
  legend_facet_label,
  legend_cum_label,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{A \code{GRanges} returned from \code{annotate_regions()}. If the data is not summarized, the data is at the region level. If it is summarized, it represents the average or standard deviation of the regions by the character vector used for \code{by} in \code{summarize_numerical()}.}

\item{x}{A string indicating the column of the \code{GRanges} to use for the x-axis.}

\item{y}{A string indicating the column of the \code{GRanges} to use for the y-axis. If missing, a a histogram over \code{x} will be plotted. If not missing, a scatterplot is plotted.}

\item{facet}{A string, or character vector of two strings, indicating indicating which categorical variable(s) in the \code{GRanges} to make \code{ggplot2} facets over. When two facets are given, the first entry is the vertical facet and the second entry is the horizontal facet. Default is \code{annot.type}.}

\item{facet_order}{A character vector, or list of character vectors if \code{facet} has length 2, which gives the order of the facets, and can be used to subset the column in the \code{GRanges} used for the \code{facet}. For example, if \code{facet = 'annot.type'}, then the annotations maybe subsetted to just CpG annotations. Default is \code{NULL}, meaning all annotations in their default order are used.}

\item{bin_width}{An integer indicating the bin width of the histogram used for score. Default 10. Select something appropriate for the data. NOTE: This is only used if \code{y} is \code{NULL}.}

\item{plot_title}{A string used for the title of the plot. If missing, no title is displayed.}

\item{x_label}{A string used for the x-axis label. If missing, no x-axis label is displayed.}

\item{y_label}{A string used for the y-axis label. If missing, no y-axis label is displayed.}

\item{legend_facet_label}{A string used to label the gray bar portion of the legend. Defaults to "x in facet".}

\item{legend_cum_label}{A string used to label the red outline portion of the legend. Defaults to "All in x".}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{ggplot} object which can be viewed by calling it, or saved with \code{ggplot2::ggsave}.
}
\description{
This function produces either histograms over \code{facet}, or x-y scatterplots over \code{facet}. In the case of histograms over facets, the All distribution (hollow histogram with red outline) is the distribution of \code{x} over all the regions in the data. The facet specific distributions (solid gray) are the distribution of \code{x} over the regions in each facet. For example, a CpG with associated percent methylation annotated to a CpG island and a promoter will count once in the All distribution, but will count once each in the CpG island and promoter facet distributions.
}
\examples{
   # An example with multi-columned data

   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
   extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
   dm_regions = read_regions(con = dm_file, extraCols = extraCols,
       rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
   dm_regions = dm_regions[1:1000]

   # Annotate the regions
   dm_annots = annotate_regions(
       regions = dm_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   # Plot histogram of group 1 methylation rates across the CpG annotations.
   # NOTE: Overall distribution (everything in \code{facet_order})
   # is plotted in each facet for comparison.
   dm_vs_regions_mu1 = plot_numerical(
       annotated_regions = dm_annots,
       x = 'mu1',
       facet = 'annot.type',
       facet_order = c('hg19_cpg_islands','hg19_cpg_shores',
           'hg19_cpg_shelves','hg19_cpg_inter'),
       bin_width = 5,
       plot_title = 'Group 1 Methylation over CpG Annotations',
       x_label = 'Group 1 Methylation')

   # Plot histogram of group 1 methylation rates across the CpG annotations
   # crossed with DM_status
   dm_vs_regions_diffmeth = plot_numerical(
       annotated_regions = dm_annots,
       x = 'diff_meth',
       facet = c('annot.type','DM_status'),
       facet_order = list(
           c('hg19_genes_promoters','hg19_genes_5UTRs','hg19_cpg_islands'),
           c('hyper','hypo','none')),
       bin_width = 5,
       plot_title = 'Group 0 Region Methylation In Genes',
       x_label = 'Methylation Difference')

   # Can also use the result of annotate_regions() to plot two numerical
   # data columns against each other for each region, and facet by annotations.
   dm_vs_regions_annot = plot_numerical(
       annotated_regions = dm_annots,
       x = 'mu0',
       y = 'mu1',
       facet = 'annot.type',
       facet_order = c('hg19_cpg_islands','hg19_cpg_shores',
           'hg19_cpg_shelves','hg19_cpg_inter'),
       plot_title = 'Region Methylation: Group 0 vs Group 1',
       x_label = 'Group 0',
       y_label = 'Group 1')

   # Another example, but using differential methylation status as the facets.
   dm_vs_regions_name = plot_numerical(
       annotated_regions = dm_annots,
       x = 'mu0',
       y = 'mu1',
       facet = 'DM_status',
       facet_order = c('hyper','hypo','none'),
       plot_title = 'Region Methylation: Group 0 vs Group 1',
       x_label = 'Group 0',
       y_label = 'Group 1')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_cellline_from_code}
\alias{get_cellline_from_code}
\title{Function to return cell line from chromatin annotation code}
\usage{
get_cellline_from_code(code)
}
\arguments{
\item{code}{The annotation code, used in \code{build_annotations()}.}
}
\value{
A string of the cell line used in a chromatin annotation code
}
\description{
Function to return cell line from chromatin annotation code
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\docType{data}
\name{annotatr_cache}
\alias{annotatr_cache}
\title{A global-variable to hold custom annotations loaded in an R session}
\format{
An object of class \code{list} of length 3.
}
\usage{
annotatr_cache
}
\value{
An environment to contain custom annotations from \code{read_annotations}.
}
\description{
Code thanks to Martin Morgan. This is a global variable that will store custom
annotations that a user reads in during a session in which annotatr is loaded.
}
\examples{
 # Example usage
 annotatr_cache$set("foo", 1:10)
 annotatr_cache$get("foo")

 # Read in a BED3 file as a custom annotation
 file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
 # The custom annotation is added to the annotatr_cache environment in this function
 read_annotations(con = file, name = 'test', genome = 'hg19')
 # The result of read_annotations() is not visible in .GlobalEnv, instead
 # need to use the get method
 print(annotatr_cache$get('hg19_custom_test'))
 # See what is in the annotatr_cache
 annotatr_cache$list_env()
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotatr_package_doc.R
\docType{package}
\name{annotatr}
\alias{annotatr-package}
\alias{annotatr}
\title{annotatr: Annotation of Genomic Regions to Functional Annotations}
\description{
Given a set of genomic sites/regions (e.g. ChIP-seq peaks, CpGs, differentially methylated CpGs or regions, SNPs, etc.) it is often of interest to investigate the intersecting functional annotations. Such annotations include those relating to gene models (promoters, 5'UTRs, exons, introns, and 3'UTRs), CpGs (CpG islands, CpG shores, CpG shelves), the non-coding genome, and enhancers. The annotatr package provides an easy way to summarize and visualize the intersection of genomic sites/regions with the above functional annotations.
}
\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://www.github.com/rcavalcante/annotatr/issues}
}

}
\author{
\strong{Maintainer}: Raymond G. Cavalcante \email{rcavalca@umich.edu}

Other contributors:
\itemize{
  \item Maureen A. Sartor \email{sartorma@med.umich.edu} [thesis advisor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_lncrna_annots}
\alias{build_lncrna_annots}
\title{A helper function to build lncRNA annotations.}
\usage{
build_lncrna_annots(genome = c("hg19", "hg38", "mm10"))
}
\arguments{
\item{genome}{The genome assembly.}
}
\value{
A \code{GRanges} object with \code{id} giving the \code{transcript_type} from the GENCODE file, \code{tx_id} being the Ensembl transcript ID, \code{gene_id} being the Entrez ID coming from a mapping of gene symbol to Entrez ID, \code{symbol} being the gene_name from the GENCODE file, and the \code{type} being \code{[genome]_lncrna_gencode}.
}
\description{
Using the \code{AnnotationHub} package, retrieve transcript level lncRNA annotations for either human (GRCh38) or mouse (GRCm38). If the genome is 'hg19', use the permalink from GENCODE and \code{rtracklayer::import()} to download and process.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarize_numerical}
\alias{summarize_numerical}
\title{Summarize numerical data over groupings of annotated regions}
\usage{
summarize_numerical(
  annotated_regions,
  by = c("annot.type", "annot.id"),
  over,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{by}{A character vector of the columns of \code{as.data.frame(annotated_regions)} to group over. Default is \code{c(annot.type, annot.id)}.}

\item{over}{A character vector of the numerical columns in \code{as.data.frame(annotated_regions)} to \code{count}, take the \code{mean}, and take the \code{sd} over after grouping according to the \code{by} column. NOTE: If more than one value is used, the naming scheme for the resuling \code{dplyr::tbl} summary columns are \code{COLNAME_n}, \code{COLNAME_mean}, \code{COLNAME_sd}. If \code{over} has length one, then the column names are \code{n}, \code{mean}, \code{sd}.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A grouped \code{dplyr::tbl_df}, and the \code{count}, \code{mean}, and \code{sd} of the \code{cols} \code{by} the groupings.
}
\description{
Given a \code{GRanges} of annotated regions, summarize numerical data columns based on a grouping.
}
\details{
NOTE: We do not take the distinct values of \code{seqnames}, \code{start}, \code{end}, \code{annot.type} as in the other \code{summarize_*()} functions because in the case of a region that intersected two distinct exons, using \code{distinct()} would destroy the information of the mean of the numerical column over one of the exons, which is not desirable.
}
\examples{
### Test on a very simple bed file to demonstrate different options

# Get premade CpG annotations
data('annotations', package = 'annotatr')

r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
r = read_regions(con = r_file, genome = 'hg19', extraCols = extraCols, rename_score = 'coverage')

a = annotate_regions(
       regions = r,
       annotations = annotations,
       ignore.strand = TRUE)

# Testing over normal by
sn1 = summarize_numerical(
       annotated_regions = a,
       by = c('annot.type', 'annot.id'),
       over = c('coverage', 'mu1', 'mu0'),
       quiet = FALSE)

# Testing over a different by
sn2 = summarize_numerical(
       annotated_regions = a,
       by = c('diff_exp'),
       over = c('coverage', 'mu1', 'mu0'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_txdb_name}
\alias{get_txdb_name}
\title{Function to get correct TxDb.* package name based on genome}
\usage{
get_txdb_name(genome = annotatr::builtin_genomes())
}
\arguments{
\item{genome}{A string giving the genome assembly.}
}
\value{
A string giving the name of the correct TxDb.* package name based on \code{genome}.
}
\description{
Function to get correct TxDb.* package name based on genome
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_cellline_from_shortcut}
\alias{get_cellline_from_shortcut}
\title{Function to return cell line from chromatin annotation shortcut}
\usage{
get_cellline_from_shortcut(shortcut)
}
\arguments{
\item{shortcut}{The annotation shortcut, used in \code{build_annotations()}.}
}
\value{
A string of the cell line used in a chromatin annotation shortcut
}
\description{
Function to return cell line from chromatin annotation shortcut
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{builtin_genomes}
\alias{builtin_genomes}
\title{Function returning supported TxDb.* genomes}
\usage{
builtin_genomes()
}
\value{
A character vector of genomes for supported TxDb.* packages
}
\description{
Function returning supported TxDb.* genomes
}
\examples{
builtin_genomes()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_annotations.R
\name{build_cpg_annots}
\alias{build_cpg_annots}
\title{A helper function to build CpG related annotations.}
\usage{
build_cpg_annots(
  genome = annotatr::builtin_genomes(),
  annotations = annotatr::builtin_annotations()
)
}
\arguments{
\item{genome}{The genome assembly.}

\item{annotations}{A character vector with entries of the form \code{[genome]_cpg_{islands,shores,shelves,inter}}.}
}
\value{
A list of \code{GRanges} objects.
}
\description{
Using the \code{AnnotationHub} package, extract CpG island track for the appropriate \code{genome} and construct the shores, shelves, and interCGI annotations as desired.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarize_categorical}
\alias{summarize_categorical}
\title{Summarize categorical data over groupings of annotated regions}
\usage{
summarize_categorical(
  annotated_regions,
  by = c("annot.type", "annot.id"),
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{by}{A character vector to group the data in \code{as.data.frame(annotated_regions)} by and tally over. Default is \code{c('annot.type', 'annot.id')}.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A grouped \code{dplyr::tbl_df} of the counts of groupings according to the \code{by} vector.
}
\description{
Given a \code{GRanges} of annotated regions, count the number of regions when the annotations are grouped \code{by} categorical columns.
}
\details{
If a region is annotated to multiple annotations of the same \code{annot.type}, the region will only be counted once. For example, if a region were annotated to multiple exons, it would only count once toward the exons, but if it were annotated to an exon and an intron, it would count towards both.
}
\examples{

   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   r_file = system.file('extdata', 'test_read_multiple_data_nohead.bed', package='annotatr')
   extraCols = c(pval = 'numeric', mu1 = 'integer', mu0 = 'integer', diff_exp = 'character')
   r = read_regions(con = r_file, genome = 'hg19', extraCols = extraCols, rename_score = 'coverage')

   a = annotate_regions(
       regions = r,
       annotations = annotations,
       ignore.strand = TRUE)

   sc = summarize_categorical(
       annotated_regions = a,
       by = c('annot.type', 'name'),
       quiet = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{read_annotations}
\alias{read_annotations}
\title{Read custom annotations}
\usage{
read_annotations(con, name, genome = NA, format, extraCols = character(), ...)
}
\arguments{
\item{con}{A path, URL, connection or BEDFile object. See \code{rtracklayer::import.bed()} documentation.}

\item{name}{A string for the name of the annotations to be used in the name of the object, [genome]_custom_[name]}

\item{genome}{From \code{rtracklayer::import()}: The identifier of a genome, or NA if unknown. Typically, this is a UCSC identifier like 'hg19'. An attempt will be made to derive the \code{seqinfo} on the return value using either an installed BSgenome package or UCSC, if network access is available.}

\item{format}{From \code{rtracklayer::import()}: The format of the output. If not missing, should be one of 'bed', 'bed15', 'bedGraph' or 'bedpe'. If missing and 'con' is a filename, the format is derived from the file extension. This argument is unnecessary when 'con' is a derivative of 'RTLFile'.}

\item{extraCols}{From \code{rtracklayer::import.bed()}: A character vector in the same form as 'colClasses' from 'read.table'.  It should indicate the name and class of each extra/special column to read from the BED file. As BED does not encode column names, these are assumed to be the last columns in the file. This enables parsing of the various BEDX+Y formats.}

\item{...}{Parameters to pass onto the format-specific method of \code{rtracklayer::import()}.}
}
\value{
A \code{GRanges} object stored in \code{annotatr_cache}. To view a custom annotation, do \code{annotatr_cache$get(name)}. To add a custom annotation to the set of annotations, include \code{'[genome]_custom_[name]'} in the call to \code{build_annotations()}. See example below.
}
\description{
\code{read_annotations()} is a wrapper for the \code{rtracklayer::import()} function that creates a \code{GRanges} object matching the structure of annotations built with \code{build_annotations()}. The structure is defined by \code{GRanges}, with the \code{mcols()} with names \code{c('id','gene_id','symbol','type')}.
}
\examples{

 # Read in a BED3 file as a custom annotation
 file = system.file('extdata', 'test_annotations_3.bed', package='annotatr')
 read_annotations(con = file, name = 'test', genome = 'hg19')
 build_annotations(genome = 'hg19', annotations = 'hg19_custom_test')

 print(annotatr_cache$get('hg19_custom_test'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize.R
\name{plot_coannotations}
\alias{plot_coannotations}
\title{Plot pair-wise annotations across regions}
\usage{
plot_coannotations(
  annotated_regions,
  annotation_order = NULL,
  plot_title,
  axes_label,
  quiet = FALSE
)
}
\arguments{
\item{annotated_regions}{The \code{GRanges} result of \code{annotate_regions()}.}

\item{annotation_order}{A character vector which doubles as the subset of annotations desired for plot as well as the ordering. If \code{NULL}, all annotations are displayed.}

\item{plot_title}{A string used for the title of the plot. If missing, no plot title label is displayed.}

\item{axes_label}{A string used for the axis labels. If missing, corresponding variable name used.}

\item{quiet}{Print progress messages (FALSE) or not (TRUE).}
}
\value{
A \code{ggplot} object which can be viewed by calling it, saved with \code{ggplot2::ggsave}, or edited.
}
\description{
All co-occurring annotations associated with a region are computed and displayed as a heatmap.
}
\details{
As with \code{plot_annotation()}, the number in each cell is the number of unique regions annotated to the pair of annotations.

For example, if a region is annotated to both a CpG shore and to two different exons simultaneously, the region will only be counted once in the CpG shore / exon cell. NOTE, this same region will count once in both the CpG shore and exon cells on the diagonal.
}
\examples{
   # Get premade CpG annotations
   data('annotations', package = 'annotatr')

   dm_file = system.file('extdata', 'IDH2mut_v_NBM_multi_data_chr9.txt.gz', package = 'annotatr')
   extraCols = c(diff_meth = 'numeric', mu1 = 'numeric', mu0 = 'numeric')
   dm_regions = read_regions(con = dm_file, extraCols = extraCols,
       rename_score = 'pval', rename_name = 'DM_status', format = 'bed')
   dm_regions = dm_regions[1:1000]

   dm_annots = annotate_regions(
       regions = dm_regions,
       annotations = annotations,
       ignore.strand = TRUE)

   all_order = c(
       'hg19_cpg_islands',
       'hg19_cpg_shores',
       'hg19_cpg_shelves',
       'hg19_cpg_inter')

   dm_vs_ca = plot_coannotations(
       annotated_regions = dm_annots,
       annotation_order = all_order,
       axes_label = 'Annotations',
       plot_title = 'Co-occurrence of Annotations')

}
